;;;                   -*- Mode: Lisp; Package: ARITH -*-                    ;;;
;;; ----------------------------------------------------------------------- ;;;
;;; File:         arith.lsp                                                 ;;;
;;; Description:  BOOMS arithmetic documents                                ;;;
;;; Author:       Eli Barzilay - eli@cs.bgu.ac.il                           ;;;
;;; Date:         28-12-96                                                  ;;;
;;; Package:      ARITH                                                     ;;;
;;; ----------------------------------------------------------------------- ;;;

#|

This file is part of the BOOMS system:
+-----------------------------------------------------------------------------+
|                                                                             |
| /######   /#####   /#####  /##     /##  /#####                              |
| /##///## /##///## /##///## /###   /### /##///##                        /--- |
| /##  /## /##  /## /##  /## /#### /#### /##  ///          /------/     /     |
| /######/ /##  /## /##  /## /##/####/## //#####          /      /     /      |
| /##///## /##  /## /##  /## /##//##//##  /////##        /------/ __  /       |
| /##  /## /##  /## /##  /## /## /// /##      /##   __  /  __  / /  \/        |
| /##  /## /##  /## /##  /## /##     /## /##  /##  /  \/  /  \/  \__/         |
| /######/ //#####/ //#####/ /##     /## //#####/  \__/   \__/                |
| ///////   //////   //////  ///     ///  //////                              |
|                                                                             |
|                   Booms Object Oriented Music Structures.                   |
|Eli Barzilay:                                                   Maze is Life!|
+-----------------------------------------------------------------------------+

|#


;;;-------------------------------------------------------------------------;;;
;;;  A R I T H  file                                                        ;;;
;;;-------------------------------------------------------------------------;;;

(in-package :arith)


;;;-------------------------------------------------------------------------;;;
;;;  A R I T H M E T I C - I C O N  type declaration                        ;;;
;;;-------------------------------------------------------------------------;;;

;;; Counter to get the next icon name.
(defvar *ARITHMETIC-ICON-COUNTER* nil)

;;; This can be used to force some name (for make-copy).
(defvar *ARITHMETIC-NAME-FORCE*   nil)

;;; Function to get the next icon name.
(defun GET-NEXT-ARITHMETIC-ICON-NAME ()
  (or *arithmetic-name-force*
      (format nil "#~a" (incf *arithmetic-icon-counter*))))

;;; This class is used as a base to arithmetic icons.  It should not inherit
;;; icon since it is also a base for leaf icons.
(defclass ARITHMETIC-ICON ()
  ((arith-name :accessor icon-arith-name
               :initform (get-next-arithmetic-icon-name))))

;;; This is a dirty trick to set the name slot, but make copy is not
;;; generalized to handle slots using inheritance, and the icon-counter better
;;; have the same value.  The make-copy is just-in-case.
(defmethod MAKE-COPY ((icon arithmetic-icon))
  (let ((*arithmetic-name-force* (make-copy (icon-arith-name icon))))
    (call-next-method)))

;;; Save the name slot.
(defmethod LOAD-SLOTS append ((x arithmetic-icon))
  '(arith-name))

;;; Setting the name means the document is modified (it will inherit from icon
;;; later on).
(defmethod (setf ICON-ARITH-NAME) :before (newval (icon arithmetic-icon))
  (set-modified (icon-doc icon)))

;;; The name of an arithmetic icon - show the name field.
(defmethod ICON-DESCRIBE-NAME ((icon arithmetic-icon))
  (format nil "~a \"~a\""
          (icon-class-to-name (type-of icon)) (icon-arith-name icon)))

;;; This is used to print the string with a small font - for numbers.
(defvar *ARITH-ICON-SMALL-TEXT* nil)

;;; Drawing an icon for general numbers - draw the string.
(defmethod ICON-DRAW-ICON ((icon arithmetic-icon))
  (let ((pos   #.(make-position 0 0))
        (box   #.(make-box 0 0 0 0))
        (dkred #.(make-rgb :red 128))
        (str   (icon-string icon))
        (doc   (icon-doc icon)))
    (nmake-position pos +icon-margin+ #.(+ +icon-margin+ 3))
    (nmake-box-relative-from-corner
     box (nposition+ pos (icon-pos icon))
     +win-icon-size+ #.(- +win-icon-size+ 6))
    (with-font (doc (if *arith-icon-small-text*
                      #.(make-font :modern :arial 14 '(:bold))
                      #.(make-font :modern :arial 28 '(:bold))))
      (with-foreground-color (doc dkred)
        (fill-box doc box))
      (with-foreground-color (doc yellow)
        (with-background-color (doc dkred)
          (draw-string-in-box
           doc str 0 (length str) box :center :center nil))))))


;;;-------------------------------------------------------------------------;;;
;;;  A R I T H M E T I C - O P E R A T O R  icons                           ;;;
;;;-------------------------------------------------------------------------;;;

;;; General base class for an arithmetic operator.
(defclass ARITHMETIC-OPERATOR (arithmetic-icon)
  ((operator :allocation :class :initform nil :accessor arith-operator)))

;;; Setting the operator means the document is modified (it will inherit from
;;; icon later on).
(defmethod (setf ARITH-OPERATOR) :before (newval (icon arithmetic-operator))
  (set-modified (icon-doc icon)))

;;; Forget the value in case of an operator change.
(defmethod (setf ARITH-OPERATOR) :after (newval (icon arithmetic-icon))
  (forget-value icon))

;;; Base classes for operators, these are defined with deficon, the rest
;;; inherit the icon functionality from these.

;;; Base class for operators that work on a list of arguments.
(deficon LIST-OPERATOR (nums list integer) (arithmetic-operator) ()
  (evaluate integer := (apply (arith-operator self) nums))
  (arith-expression :fullargs :evaled arith-expression
                    := (list* (arith-operator self) nums)))

;;; Base class for unary operators.
(deficon UNARY-OPERATOR ((x integer)) (arithmetic-operator) ()
  (evaluate integer := (funcall (arith-operator self) x))
  (arith-expression :fullargs :evaled arith-expression
                    := (list (arith-operator self) x)))

;;; Base class for binary operators.
(deficon BINARY-OPERATOR ((x integer) (y integer)) (arithmetic-operator) ()
  (evaluate integer := (funcall (arith-operator self) x y))
  (arith-expression :fullargs :evaled arith-expression
                    := (list (arith-operator self) x y)))

;;; Base classes for the basic arithmetic operations.
(defclass BASIC-OPERATOR (arithmetic-operator) ())

;;; The string to draw is the operator symbol.
(defmethod ICON-STRING ((b-o basic-operator))
  (symbol-name (arith-operator b-o)))

;;; Some arithmetic operators.

(defclass PLUS-OPERATOR (basic-operator)
  ((operator :allocation :class :initform '+)))

(defclass MINUS-OPERATOR (basic-operator)
  ((operator :allocation :class :initform '-)))

(defclass MULT-OPERATOR (basic-operator)
  ((operator :allocation :class :initform '*)))

(defclass INC-OPERATOR (basic-operator)
  ((operator :allocation :class :initform '1+)))

(defclass DEC-OPERATOR (basic-operator)
  ((operator :allocation :class :initform '1-)))

(defclass SQUARE-OPERATOR (basic-operator)
  ((operator :allocation :class :initform '^2)))

(defun ^2 (x) (* x x))

;;; The actual node operators.

(defclass LIST-PLUS  (plus-operator   list-operator  ) ())

(defclass BIN-PLUS   (plus-operator   binary-operator) ())

(defclass LIST-MINUS (minus-operator  list-operator  ) ())

(defclass BIN-MINUS  (minus-operator  binary-operator) ())

(defclass UNR-MINUS  (minus-operator  unary-operator ) ())

(defclass LIST-MULT  (mult-operator   list-operator  ) ())

(defclass BIN-MULT   (mult-operator   binary-operator) ())

(defclass INCREMENT  (inc-operator    unary-operator ) ())

(defclass DECREMENT  (dec-operator    unary-operator ) ())

(defclass SQUARE     (square-operator unary-operator ) ())


;;;-------------------------------------------------------------------------;;;
;;;  G E N E R A L - O P E R A T O R  icons                                 ;;;
;;;-------------------------------------------------------------------------;;;

;;; An operator with a variable symbol.
(defclass GENERAL-OPERATOR (basic-operator)
  ((operator :allocation :instance :initform nil :accessor arith-operator)))

;;; This symbol must be saved.
(defmethod MAKE-COPY ((icon general-operator))
  (let ((result (call-next-method)))
    (setf (arith-operator result) (arith-operator icon))
    result))

;;; Save the operator slot.
(defmethod LOAD-SLOTS append ((x general-operator))
  '(operator))

;;; The name of a general icon - show the operator.
(defmethod ICON-DESCRIBE-NAME ((icon general-operator))
  (format nil "~a[~a] \"~a\""
          (icon-class-to-name (type-of icon))
          (arith-operator icon) (icon-arith-name icon)))

;;; The three versions of the above, with a method to return possible
;;; operators.

;;; List operators.

(defclass GENERAL-LIST-OPERATOR (general-operator list-operator) ())

(set-booms-name general-list-operator 'list-op)

(defmethod ARITHMETIC-OPERATOR-OPTIONS ((icon general-list-operator))
  '(+ - *))

;;; Binary operators.

(defclass GENERAL-BINARY-OPERATOR (general-operator binary-operator) ())

(set-booms-name general-binary-operator 'binary-op)

(defmethod ARITHMETIC-OPERATOR-OPTIONS ((icon general-binary-operator))
  '(+ - * % ^))

(defun % (x y) (mod x y))

(defun ^ (x y) (expt x y))

;;; Unary operators.

(defclass GENERAL-UNARY-OPERATOR (general-operator unary-operator) ())

(set-booms-name general-unary-operator 'unary-op)

(defmethod ARITHMETIC-OPERATOR-OPTIONS ((icon general-unary-operator))
  '(- 1+ 1- ^2 sq))

(defun SQ (x) (isqrt x))

;;; Ask user for initial value using the operator pane.
(defmethod ICON-INITIALIZE ((icon general-operator))
  (setf (arith-operator icon) (first (arithmetic-operator-options icon)))
  (icon-edit icon nil :operator t))


;;;-------------------------------------------------------------------------;;;
;;;  G E N E R A L - N U M B E R  leaves                                    ;;;
;;;-------------------------------------------------------------------------;;;

;;; A general number class.
(defleaf GENERAL-NUMBER (arithmetic-icon)
  ((type-out :allocation :class :initform 'integer)))

(set-booms-name general-number 'number)

;;; Ask user for initial value using the operator pane.
(defmethod LEAF-INIT-VAL/PANE ((icon general-number))
  (values 0 :leaf))

;;; The lisp expression is simply the number.
(defmethod ARITH-EXPRESSION ((icon general-number))
  (arguments icon))

;;; The string is the number.
(defmethod ICON-STRING ((icon general-number))
  (princ-to-string (arguments icon)))

;;; Do this using a small font.
(defmethod ICON-DRAW-ICON ((icon general-number))
  (let ((*arith-icon-small-text* t))
    (call-next-method)))


;;;-------------------------------------------------------------------------;;;
;;;  C O N S T A N T  number leaves                                         ;;;
;;;-------------------------------------------------------------------------;;;

;;; Constant number icons.
(defconstleaf CONSTANT-NUMBER (general-number) ())

;;; Macro for a definition of a constant numeral leaf.
(defmacro DEFCONSTNUMERAL (sym name val)
  `(progn
     (defleaf ,sym (constant-number)
       ((arguments :initform ,val)
        (str       :reader     icon-string
                   :allocation :class
                   :initform   (format nil "~:(~r~)" ,val))))
     (set-booms-name ,sym ,name)))

;;; Constant digits for 0 ... 10.

(defconstnumeral ZERO-CONST  'zero  0)

(defconstnumeral ONE-CONST   'one   1)

(defconstnumeral TWO-CONST   'two   2)

(defconstnumeral THREE-CONST 'three 3)

(defconstnumeral FOUR-CONST  'four  4)

(defconstnumeral FIVE-CONST  'five  5)

(defconstnumeral SIX-CONST   'six   6)

(defconstnumeral SEVEN-CONST 'seven 7)

(defconstnumeral EIGHT-CONST 'eight 8)

(defconstnumeral NINE-CONST  'nine  9)

(defconstnumeral TEN-CONST   'ten   10)


;;;-------------------------------------------------------------------------;;;
;;;  A R I T H M E T I C  icon menu                                         ;;;
;;;-------------------------------------------------------------------------;;;

;;; The menu variable.
(defvar *ARITHMETIC-ICON-MENU* nil)

;;; Return this menu for arithmetic icons.
(defmethod ICON-MENU ((icon arithmetic-icon))
  *arithmetic-icon-menu*)

;;; Create the menu.
(defun MAKE-ARITHMETIC-ICON-MENU ()
  (let ((menu (make-default-icon-menu)))
    (add-icon-menu-item menu 'show-exp "~Show Expression"
                        "Show the Lisp expression for this icon")
    menu))

;;; Menu handlers.

;;; Show the Lisp expression for some arithmetic icon.
(defmethod SHOW-EXP ((icon arithmetic-icon))
  (say "The Lisp expression for \"~a\" is ~a"
       (icon-arith-name icon) (arith-expression icon)))

;;; Do the above for standard icons & expression for .

(defmethod ICON-ARITH-NAME ((icon identity))
  "Identity")

(defmethod ARITH-EXPRESSION ((obj t))
  obj)

(defmethod ARITH-EXPRESSION ((icon identity))
  (arith-expression (getarg0 icon 0)))

(defmethod ARITH-EXPRESSION ((icon abstraction))
  (arith-expression (root-node icon)))


;;;-------------------------------------------------------------------------;;;
;;;  N A M E  &  O P E R A T O R  edit pages                                ;;;
;;;-------------------------------------------------------------------------;;;

;;; The name page.

;;; The arith-name edit box widget.
(defvar *ARITH-NAME-TEXT* nil)

(defun MAKE-ARITH-NAME-TAB ()
  (add-icon-edit-tab :arith-name)
  (progrev
   (add-icon-edit-tab-widget
     :widget       'static-text
     :value        "~Name"
     :box          (make-box 40 30 200 50)
     :tabstop      nil
     :groupstart   t)
   (setf *arith-name-text*
    (add-icon-edit-tab-widget
     :widget       'editable-text
     :value        nil
     :box          (make-box 40 60 200 90)
     :3d-border    t
     :tabstop      t
     :groupstart   nil
     :set-value-fn #'true))
   (add-icon-edit-tab-widget
     :widget       'button
     :title        "~A p p l y"
     :box          (make-box 40 100 200 130)
     :3d-border    t
     :tabstop      t
     :groupstart   nil
     :set-value-fn #'(lambda (&rest r)
                       (let ((newval (dialog-item-value *arith-name-text*)))
                         (setf (icon-arith-name *icon-edit-icon*) newval)
                         ;; No need for redraw, since the name is not drawn.
                         nil)))))

;;; Initialize the page values.
(defmethod ICON-EDIT-SETUP-PAGE ((icon arithmetic-icon) (id (eql :arith-name))
                                 &optional socket)
  (set-dialog-item-value *arith-name-text* (icon-arith-name icon)))

;;; Terminate the page.
(defmethod ICON-EDIT-DONE-PAGE ((icon arithmetic-icon) (id (eql :arith-name))
                                result)
  (when result
    (setf (icon-arith-name icon) (dialog-item-value *arith-name-text*))))

;;; Return this page for arithmetic-icon.
(defmethod ICON-TAB-IDS ((x arithmetic-icon))
  (list* :arith-name (call-next-method)))

;;; The operator page.

;;; The operator combo-box widget.
(defvar *OPERATOR-COMBO-BOX* nil)

(defun MAKE-OPERATOR-TAB ()
  (add-icon-edit-tab :operator)
  (progrev
   (add-icon-edit-tab-widget
     :widget       'static-text
     :value        "~Operator"
     :box          (make-box 40 30 200 50)
     :tabstop      nil
     :groupstart   t)
   (setf *operator-combo-box*
    (add-icon-edit-tab-widget
     :widget       'combo-box
     :value        nil
     :range        nil
     :key          'capitalize-object
     :box          (make-box 40 60 200 240)
     :3d-border    t
     :tabstop      t
     :groupstart   nil
     :set-value-fn #'true))
   (add-icon-edit-tab-widget
     :widget       'button
     :title        "~A p p l y"
     :box          (make-box 40 100 200 130)
     :3d-border    t
     :tabstop      t
     :groupstart   nil
     :set-value-fn #'(lambda (&rest r)
                       (let ((newval (dialog-item-value *operator-combo-box*)))
                         (setf (arith-operator *icon-edit-icon*) newval)
                         (do-redraws (icon-doc *icon-edit-icon*))
                         nil)))))

;;; Initialize the page values.
(defmethod ICON-EDIT-SETUP-PAGE ((icon general-operator) (id (eql :operator))
                                 &optional socket)
  (set-dialog-item-range *operator-combo-box* nil)
  (dolist (op (arithmetic-operator-options icon))
    (list-widget-add-item *operator-combo-box* op))
  (set-dialog-item-value *operator-combo-box* (arith-operator icon)))

;;; Terminate the page.
(defmethod ICON-EDIT-DONE-PAGE ((icon general-operator) (id (eql :operator))
                                result)
  (when result
    (setf (arith-operator icon) (dialog-item-value *operator-combo-box*))))

;;; Return this page for general-operator.
(defmethod ICON-TAB-IDS ((x general-operator))
  (list* :operator (call-next-method)))


;;;-------------------------------------------------------------------------;;;
;;;  A R I T H M E T I C  package initialization                            ;;;
;;;-------------------------------------------------------------------------;;;

(defun INITIALIZE-ARITHMETIC-PACKAGE ()
  (register-type :arithmetic)
  (make-operator-tab)
  (make-arith-name-tab)
  (setf *arithmetic-icon-menu* (make-arithmetic-icon-menu))
  (add-node-class 'list-plus               :list-ops   )
  (add-node-class 'bin-plus                :binary-ops )
  (add-node-class 'list-minus              :list-ops   )
  (add-node-class 'bin-minus               :binary-ops )
  (add-node-class 'unr-minus               :unary-ops  )
  (add-node-class 'list-mult               :list-ops   )
  (add-node-class 'bin-mult                :binary-ops )
  (add-node-class 'increment               :unary-ops  )
  (add-node-class 'decrement               :unary-ops  )
  (add-node-class 'square                  :unary-ops  )
  (add-node-class 'general-list-operator   :general-ops)
  (add-node-class 'general-binary-operator :general-ops)
  (add-node-class 'general-unary-operator  :general-ops)
  (add-node-class 'general-number          :numbers    )
  (add-node-class 'zero-const              :numbers    )
  (add-node-class 'one-const               :numbers    )
  (add-node-class 'two-const               :numbers    )
  (add-node-class 'three-const             :numbers    )
  (add-node-class 'four-const              :numbers    )
  (add-node-class 'five-const              :numbers    )
  (add-node-class 'six-const               :numbers    )
  (add-node-class 'seven-const             :numbers    )
  (add-node-class 'eight-const             :numbers    )
  (add-node-class 'nine-const              :numbers    )
  (add-node-class 'ten-const               :numbers    )
  (setf *arithmetic-icon-counter* 0
        *arithmetic-name-force*   nil
        *arith-icon-small-text*   nil))


;;;-------------------------------------------------------------------------;;;
;;;  D O N E !                                                              ;;;
;;;-------------------------------------------------------------------------;;;

(provide :ARITH)

