;;;                  -*- Mode: Lisp; Package: CL-USER -*-                   ;;;
;;; ----------------------------------------------------------------------- ;;;
;;; File:         b-files.lsp                                               ;;;
;;; Description:  BOOMS lisp make file                                      ;;;
;;; Author:       Eli Barzilay - eli@cs.bgu.ac.il                           ;;;
;;; Date:         5-10-96                                                   ;;;
;;; Package:      CL-USER                                                   ;;;
;;; ----------------------------------------------------------------------- ;;;

#|

This file is part of the BOOMS system:
+-----------------------------------------------------------------------------+
|                                                                             |
| /######   /#####   /#####  /##     /##  /#####                              |
| /##///## /##///## /##///## /###   /### /##///##                        /--- |
| /##  /## /##  /## /##  /## /#### /#### /##  ///          /------/     /     |
| /######/ /##  /## /##  /## /##/####/## //#####          /      /     /      |
| /##///## /##  /## /##  /## /##//##//##  /////##        /------/ __  /       |
| /##  /## /##  /## /##  /## /## /// /##      /##   __  /  __  / /  \/        |
| /##  /## /##  /## /##  /## /##     /## /##  /##  /  \/  /  \/  \__/         |
| /######/ //#####/ //#####/ /##     /## //#####/  \__/   \__/                |
| ///////   //////   //////  ///     ///  //////                              |
|                                                                             |
|                   Booms Object Oriented Music Structures.                   |
|Eli Barzilay:                                                   Maze is Life!|
+-----------------------------------------------------------------------------+

|#


(in-package :cl-user)


;;;-------------------------------------------------------------------------;;;
;;;  B O O M S  files                                                       ;;;
;;;-------------------------------------------------------------------------;;;

;;; The BOOMS packages definition file.
(defconstant +BOOMS-PACKAGES-FILE+ "packages")

;;; These variables should be set when loading the package file.
(defvar *BOOMS-PACKAGE* nil)
(defvar *BOOMS-FILES*   nil)

;;; The lisp file and fasl file extensions.
(defconstant +LISP-EXTENSION+ "lsp")
(defconstant +FASL-EXTENSION+ "fsl")

;;; Functions that will use the above to return a filename.

(defun LISP-FILENAME (fname)
  (concatenate 'string fname "." +lisp-extension+))

(defun FASL-FILENAME (fname)
  (concatenate 'string fname "." +fasl-extension+))

;;; The list of files loaded and time of loaded files.
(defvar *BOOMS-LOADED-FILES*
  (list
   ;; Make this file first so it can be used with load-file itself.
   (cons (pathname-name *load-pathname*) (file-write-date *load-pathname*))))

;;; Format a message on the lisp screen and on the status bar.
;;; newlines is one of ":--", ":+-", ":-+" or ":++".
(defun FORMAT-MESSAGE (newlines formatstr &rest args)
  (let ((str (apply #'format nil formatstr args)))
    (window-message *lisp-frame-window* str)
    (format t "~:[~;~&~]~a~:[~;~%~]"
            (member newlines '(:+- :++))
            str
            (member newlines '(:-+ :++)))))

;;; Load a file - use most updated version, do not load if already loaded.
(defun LOAD-FILE (filename)
  (let* ((fname (pathname-name filename))
         (lisp-filename (lisp-filename fname))
         (fasl-filename (fasl-filename fname))
         (lisp-file-t   (file-write-date lisp-filename))
         (fasl-file-t   (file-write-date fasl-filename))
         (best (cond ((and (not lisp-file-t) (not fasl-file-t))
                      nil)
                     ((not lisp-file-t)
                      (format t "~&Warning: No source for '~a'~%"
                              fasl-filename)
                      (cons fasl-filename fasl-file-t))
                     ((not fasl-file-t)
                      (cons lisp-filename lisp-file-t))
                     ((< lisp-file-t fasl-file-t)
                      (cons fasl-filename fasl-file-t))
                     (t
                      (cons lisp-filename lisp-file-t))))
         (loaded (assoc fname *booms-loaded-files* :test #'equalp))
         (package (rest (assoc fname *booms-files* :test #'equalp))))
    (if best
      (if (or (not loaded) (not (= (rest loaded) (rest best))))
        (progn
          (format-message
           :+- "Loading '~a' (~:[packages file~;~:*~a package~])... "
           (first best) package)
          (load (first best) :verbose nil)
          (format-message :-+ "Done.")
          (if loaded
            (setf (rest loaded) (rest best))
            (push (cons fname (rest best)) *booms-loaded-files*)))
        #+ignore (format-message :++ "File '~a' already loaded." (first best)))
      (error "Could not find file '~a' in any form." fname))))

;;; This will load all BOOMS files.
(defun LOAD-BOOMS-FILES ()
  (load-file +booms-packages-file+)
  (with-hourglass
    (dolist (f (mapcar #'first *booms-files*))
      (load-file f))))

;;; This will compile all BOOMS files.
(defun COMPILE-BOOMS-FILES ()
  (load-booms-files)
  (dolist (f (cons +booms-packages-file+ (mapcar #'first *booms-files*)))
    (when (and (probe-file (lisp-filename f))
               (or (not (probe-file (fasl-filename f)))
                   (<= (file-write-date (fasl-filename f))
                       (file-write-date (lisp-filename f)))))
      (format-message :++ "Compiling ~:@(~a.~a~)..." f +lisp-extension+)
      (compile-file (lisp-filename f)
                    :output-file (fasl-filename f)
                    :verbose nil :print nil)
      (setf (rest (assoc f *booms-loaded-files* :test #'equalp))
            (file-write-date (fasl-filename f)))
      (format-message :++ "Done."))))

;;; Flag - true when BOOMS is already loaded.
(defvar *BOOMS-LOADED* nil)
;;; The BOOMS header text to printed on the lisp screen.
(defvar *BOOMS-TEXT*   nil)

;;; When finished loading - print a message and on first loading - show header.
(defun BOOMS-DONE-LOADING/COMPILING (action)
  (if *booms-loaded*
    (format-message :++ "BOOMS re~a done." action)
    (progn
      #+ignore (clear-page t)
      (with-open-file (file "header" :direction :input)
        (setf *booms-text*
              (with-output-to-string (str)
                (let ((eof nil))
                  (while (not eof)
                    (multiple-value-bind (s eof-p) (read-line file)
                      (princ s str)
                      (terpri str)
                      (setf eof eof-p)))))))
      (princ *booms-text*)
      (with-open-file (file "booms.txt" :direction :input)
        (setf *booms-text*
              (concatenate 'string *booms-text*
                           (with-output-to-string (str)
                             (let ((eof nil))
                               (while (not eof)
                                 (multiple-value-bind
                                     (s eof-p) (read-line file)
                                   (princ s str)
                                   (terpri str)
                                   (setf eof eof-p))))))))
      (setf *booms-loaded* t)))
  (values))


;;;-------------------------------------------------------------------------;;;
;;;  D O N E !                                                              ;;;
;;;-------------------------------------------------------------------------;;;

(provide :B-FILES)

