;;;                 -*- Mode: Lisp; Package: BOOMS-APP -*-                  ;;;
;;; ----------------------------------------------------------------------- ;;;
;;; File:         classpal.lsp                                              ;;;
;;; Description:  BOOMS classes widgets and palette                         ;;;
;;; Author:       Eli Barzilay - eli@cs.bgu.ac.il                           ;;;
;;; Date:         8-10-96                                                   ;;;
;;; Package:      BOOMS-APP                                                 ;;;
;;; ----------------------------------------------------------------------- ;;;

#|

This file is part of the BOOMS system:
+-----------------------------------------------------------------------------+
|                                                                             |
| /######   /#####   /#####  /##     /##  /#####                              |
| /##///## /##///## /##///## /###   /### /##///##                        /--- |
| /##  /## /##  /## /##  /## /#### /#### /##  ///          /------/     /     |
| /######/ /##  /## /##  /## /##/####/## //#####          /      /     /      |
| /##///## /##  /## /##  /## /##//##//##  /////##        /------/ __  /       |
| /##  /## /##  /## /##  /## /## /// /##      /##   __  /  __  / /  \/        |
| /##  /## /##  /## /##  /## /##     /## /##  /##  /  \/  /  \/  \__/         |
| /######/ //#####/ //#####/ /##     /## //#####/  \__/   \__/                |
| ///////   //////   //////  ///     ///  //////                              |
|                                                                             |
|                   Booms Object Oriented Music Structures.                   |
|Eli Barzilay:                                                   Maze is Life!|
+-----------------------------------------------------------------------------+

|#


;;;-------------------------------------------------------------------------;;;
;;;  C L A S S P A L  file                                                  ;;;
;;;-------------------------------------------------------------------------;;;

(in-package :booms-app)


;;;-------------------------------------------------------------------------;;;
;;;  P A L E T T E  definitions                                             ;;;
;;;-------------------------------------------------------------------------;;;

;;; This will be the palette window.
(defvar *CLASSES-PALETTE*       nil)

;;; Is it shown?
(defvar *CLASSES-PALETTE-SHOWN* nil)

;;; Draggable widgets is a base class for the classes outline that can be
;;; dragged.

(defclass DRAGGABLE-WIDGET () ())

(defmethod DRAGGABLE-P ((drag-widget draggable-widget))
  (if *prevent-actions* nil t))

;;; The draggable outline class.
(defclass DRAGGABLE-OUTLINE (draggable-widget outline) ())

;;; The float palette is a dialog that simply transfer focus to the BOOMS pane.

(defclass FLOAT-PALETTE (dialog) ())

(defmethod EVENT ((w float-palette) event buttons data time)
  (select event
    (mouse-left-up   (call-next-method) (select-window *booms-app*))
    (mouse-left-down (call-next-method) (select-window *booms-app*))
    (t (call-next-method))))

(defmethod NC-EVENT ((w float-palette) event buttons data time section)
  (select event
    (nc-mouse-left-up   (call-next-method) (select-window *booms-app*))
    (nc-mouse-left-down (call-next-method) (select-window *booms-app*))
    (t (call-next-method))))

;;; Showing / hiding the palette.

(defun SHOW-PALETTE ()
  (setf *classes-palette-shown* t)
  (setf (window-state *classes-palette*) :normal))

(defun HIDE-PALETTE ()
  (setf *classes-palette-shown* nil)
  (setf (window-state *classes-palette*) :hidden))

(defun TOGGLE-PALETTE ()
  (if *classes-palette-shown* (hide-palette) (show-palette)))

;;; Instead of closing - just hide the palette window.
(defmethod USER-CLOSE ((win float-palette))
  (hide-palette)
  t)

;;; Create the pallete window with the outline widget.
(defun INITIALIZE-CLASS-PALETTE ()
  (setf *classes-palette*
        (open-dialog
         (list
          (setf
           *booms-icon-class-outline*
            (make-dialog-item
             :widget             'draggable-outline
             :range              nil
             :tabstop            t
             :groupstart         t
             :help-string        (format
                                  nil
                                  "Classes Palette - Select a class for ~
                                   icons creation    [F10 to toggle]")
             :key                #'(lambda (x)
                                     (capitalize-object
                                      (icon-class-to-name x)))
             :user-scrollable    t
             :left-attachment    :left
             :right-attachment   :right
             :top-attachment     :top
             :bottom-attachment  :bottom
             :set-value-fn       #'(lambda (listbox newitem olditem)
                                     (cond
                                       (*called-programmatically* t)
                                       (*prevent-actions*
                                        (select-window *booms-app*)
                                        nil)
                                       ((member newitem *node-classes*)
                                        (set-current-node-class newitem) t)
                                       (t nil)))
             :mouse-in-fn        #'show-help-text
             :mouse-out-fn       #'unsay
             :draw-icons-p       nil
             :box (make-box 0 0 +palette-size-x+ +palette-size-y+))))
         'float-palette *booms-app*
         :title            "Classes"
         :window-border    :palette
         :window-state     :shrunk
         :font             +palette-font+
         :user-closable    t
         :user-moveable    t
         :user-resizable   t
         :user-scrollable  nil
         :user-shrinkable  nil
         :overlapped       t
         :pop-up-p         nil
         :window-exterior
         (let* ((pos (nnbox-top-right (window-interior *booms-app*))))
           (nposition+ pos (nnbox-top-left (window-exterior (get-booms-pane))))
           (make-box
            (- (position-x pos) +palette-size-x+ 8)
            (+ (position-y pos) (get-caption-height))
            (position-x pos)
            (+ (position-y pos) +palette-size-y+ 24 (get-caption-height))))))
  (hide-palette))

;;; Refresh the display - doing redraws and let scroll bars appear.
(defun REFRESH-CLASS-PALETTE ()
  (let ((pos   #.(make-position 0 0))
        (x     (window-interior-width  *classes-palette*))
        (y     (window-interior-height *classes-palette*))
        (shown *classes-palette-shown*))
    (when shown
      (hide-palette)
      ;; This is for the scroll-bar to appear.
      (resize-window *classes-palette* (nmake-position pos x (1+ y)))
      (resize-window *classes-palette* (nmake-position pos x y))
      ;; Scroll the selected value into view.
      (let ((*called-programmatically* t)
            (value (dialog-item-value *booms-icon-class-outline*)))
        (set-dialog-item-value *booms-icon-class-outline* nil)
        (set-dialog-item-value *booms-icon-class-outline* value)
        (pc::outline-scroll-current-index-into-view
         *booms-icon-class-outline*))
      (show-palette))))

;;; This does the creation of an icon - it is always a bad drop since values
;;; cannot be dragged to a window pane.
(defmethod BAD-DROP ((drag-widget draggable-widget) (dragged-over null)
                     (drag-pos position))
  (let* ((doc (active-document))
         (pos (and doc (cursor-position doc)))
         (*prevent-actions* t))
    (when (and doc
               (inside-box-p pos (visible-box doc))
               (not (inside-box-p (cursor-position *classes-palette*)
                                  (visible-box *classes-palette*))))
      (create-booms-icon doc pos t)
      (setf *prevent-next-ready-message* t))
    (select-window *booms-app*)))


;;;-------------------------------------------------------------------------;;;
;;;  N O D E - C L A S S E S  handling                                      ;;;
;;;-------------------------------------------------------------------------;;;

;;; Addition.

;;; Added a node class, handle menu items.  Called during initialization when
;;; *registering-type* is bound, and during named abstraction creation when
;;; *registering-type* is not bound.  The path should be a list of symbols,
;;; going upwards - from the least to the most specific.  *registering-type*
;;; value of t is only used for disabling the setting of the current class.
(defun ADD-NODE-CLASS (new-class &rest path)
  (let ((path (if (and *registering-type* (not (eq *registering-type* t)))
                (cons *registering-type* path) path))
        (menu-item
         (make-menu-item :title       (capitalize-object
                                       (icon-class-to-name new-class))
                         :value       #'(lambda ()
                                          (set-current-node-class new-class))
                         :plist       (list :class new-class)
                         :help-string (format nil "Create \'~:(~a~)\' icons"
                                              new-class)))
        (outline-item (make-outline-item new-class nil :open ())))
    (unless (member new-class *node-classes*)
      (pushlast new-class *node-classes*))
    (push menu-item *booms-icon-class-menu-items*)
    (add-to-menu-in-path    *booms-icon-class-menu*    path menu-item)
    (add-to-outline-in-path *booms-icon-class-outline* path outline-item)
    (cond
      ((not *registering-type*)
       (set-current-node-class new-class t))
      ((eq *registering-type* t)
       (refresh-class-palette)))))

;;; Add a menu item at a given hierarchical place inside the menu.
(defun ADD-TO-MENU-IN-PATH (menu path newitem)
  (if (null path)
    (add-to-menu menu newitem)
    (let* ((obj (first path))
           (mi (find obj (menu-items menu)
                     :key #'menu-item-title :test #'equalp)))
      (if mi
        (add-to-menu-in-path (menu-item-value mi) (rest path) newitem)
        (let* ((new-menu (open-menu nil 'my-pull-down-menu *screen*
                                    :selection-function 'funcall-menu-item))
               (menu-item
                (make-menu-item :title obj
                                :value new-menu
                                :help-string
                                (format nil "~:(~a~) classes submenu" obj))))
          (add-to-menu menu menu-item)
          (add-to-menu-in-path new-menu (rest path) newitem))))))

;;; Add an outline item at a given hierarchical place inside the outline.
(defun ADD-TO-OUTLINE-IN-PATH (outline path newitem)
  (if (null path)
    (outline-add-item outline newitem nil :position :end)
    (let ((item (find (first path) (dialog-item-range outline)
                      :key #'outline-item-value :test #'equalp)))
      (unless item
        (outline-add-item outline
                          (make-outline-item (first path) nil :open ()
                                             :foreground-color red)
                          nil :position :end)
        (setf item (find (first path) (dialog-item-range outline)
                         :key #'outline-item-value :test #'equalp)))
      (do ((path (rest path) (rest path)))
          ((null path))
        (let ((subitem (find (first path) (outline-item-subitems item)
                             :key #'outline-item-value :test #'equalp)))
          (unless subitem
            (setf subitem (make-outline-item (first path) nil :open ()
                                             :foreground-color red))
            (pushlast subitem (outline-item-subitems item)))
          (setf item subitem)))
      (pushlast newitem (outline-item-subitems item)))))

;;; Rename.

;;; Rename a node class, the oldclass will be part of the path.
(defun RENAME-NODE-CLASS (oldclass newclass &rest path)
  (rename-menu-item-in-path
   *booms-icon-class-menu*    (append path (list oldclass)) newclass)
  (rename-outline-item-in-path
   *booms-icon-class-outline* (append path (list oldclass)) newclass))

;;; Rename a menu item at a given hierarchical place inside the menu.  The last
;;; path element is the old value (it is never null).
(defun RENAME-MENU-ITEM-IN-PATH (menu path newval)
  (if (null path)
    (progn
      (setf (menu-item-title menu) (capitalize-object newval))
      (setf (menu-item-value menu) #'(lambda ()
                                       (set-current-node-class newval)))
      (setf (menu-item-help-string menu)
            (format nil "Create \'~:(~a~)\' icons" newval)))
    (let* ((obj (first path))
           (mi (find obj (menu-items menu)
                     :key #'menu-item-title :test #'equalp)))
      (rename-menu-item-in-path (if (null (rest path)) mi (menu-item-value mi))
                                (rest path) newval))))

;;; Rename an outline item at a given hierarchical place inside the outline.
;;; The last path element is the old value (it is never null).
(defun RENAME-OUTLINE-ITEM-IN-PATH (outline path newval)
  (let ((item (find (first path) (dialog-item-range outline)
                    :key #'outline-item-value :test #'equalp)))
    (do ((path (rest path) (rest path)))
        ((null path))
      (setf item (find (first path) (outline-item-subitems item)
                       :key #'outline-item-value :test #'equalp)))
    (setf (outline-item-value item) newval)))

;;; Delete.

;;; Delete a node class.
(defun DELETE-NODE-CLASS (class &rest path)
  (delete-menu-item-in-path    *booms-icon-class-menu*    class path)
  (delete-outline-item-in-path *booms-icon-class-outline* class path))

;;; Delete a menu item at a given hierarchical place inside the menu.
(defun DELETE-MENU-ITEM-IN-PATH (menu name path)
  (if (null path)
    (remove-from-menu
     menu (find name (menu-items menu) :key #'menu-item-title
                :test #'equalp :test #'equalp))
    (let* ((obj (first path))
           (mi (find obj (menu-items menu)
                     :key #'menu-item-title :test #'equalp)))
      (delete-menu-item-in-path (menu-item-value mi) name (rest path)))))

;;; Delete an outline item at a given hierarchical place inside the outline.
;;; The last path element is the old value (it is never null).
(defun DELETE-OUTLINE-ITEM-IN-PATH (outline name path)
  (if (null path)
    (set-dialog-item-range outline
                           (remove name (dialog-item-range outline)
                                   :key #'outline-item-value :test #'equalp))
    (let ((item (find (first path) (dialog-item-range outline)
                      :key #'outline-item-value :test #'equalp)))
      (do ((path (rest path) (rest path)))
          ((null path))
        (setf item (find (first path) (outline-item-subitems item)
                         :key #'outline-item-value :test #'equalp)))
      (setf (outline-item-subitems item)
            (delete name (outline-item-subitems item)
                    :key #'outline-item-value :test #'equalp)))))

;;; Selecting a new node class.
(defun SET-CURRENT-NODE-CLASS (new-class &optional refresh)
  (setf *current-node-class* new-class)
  (when new-class
    (say "Current Node is now ~:(~a~)" (icon-class-to-name new-class)))
  (let ((*called-programmatically* t))
    (dolist (mi *booms-icon-class-menu-items*)
      (set-menu-item-selected-p
       mi (eq (getf (menu-item-plist mi) :class) new-class)))
    (set-dialog-item-value *booms-icon-class-outline* new-class))
  (when refresh
    (refresh-class-palette)))

;;; Select the first available node for some top-level type.
(defun SET-FIRST-AVAILABLE-NODE-CLASS-FOR (name &optional refresh)
  (let ((item (find name (dialog-item-range *booms-icon-class-outline*)
                    :key #'outline-item-value :test #'equalp)))
    (while (outline-item-subitems item)
      (setf item (first (outline-item-subitems item))))
    (set-current-node-class (outline-item-value item) refresh)))


;;;-------------------------------------------------------------------------;;;
;;;  D O N E !                                                              ;;;
;;;-------------------------------------------------------------------------;;;

(provide :ABSTRACT)

