;;;                 -*- Mode: Lisp; Package: BOOMS-APP -*-                  ;;;
;;; ----------------------------------------------------------------------- ;;;
;;; File:         dialogs.lsp                                               ;;;
;;; Description:  BOOMS lisp bitmaps                                        ;;;
;;; Author:       Eli Barzilay - eli@cs.bgu.ac.il                           ;;;
;;; Date:         19-12-96                                                  ;;;
;;; Package:      BOOMS-APP                                                 ;;;
;;; ----------------------------------------------------------------------- ;;;

#|

This file is part of the BOOMS system:
+-----------------------------------------------------------------------------+
|                                                                             |
| /######   /#####   /#####  /##     /##  /#####                              |
| /##///## /##///## /##///## /###   /### /##///##                        /--- |
| /##  /## /##  /## /##  /## /#### /#### /##  ///          /------/     /     |
| /######/ /##  /## /##  /## /##/####/## //#####          /      /     /      |
| /##///## /##  /## /##  /## /##//##//##  /////##        /------/ __  /       |
| /##  /## /##  /## /##  /## /## /// /##      /##   __  /  __  / /  \/        |
| /##  /## /##  /## /##  /## /##     /## /##  /##  /  \/  /  \/  \__/         |
| /######/ //#####/ //#####/ /##     /## //#####/  \__/   \__/                |
| ///////   //////   //////  ///     ///  //////                              |
|                                                                             |
|                   Booms Object Oriented Music Structures.                   |
|Eli Barzilay:                                                   Maze is Life!|
+-----------------------------------------------------------------------------+

|#


;;;-------------------------------------------------------------------------;;;
;;;  D I A L O G S  file                                                    ;;;
;;;-------------------------------------------------------------------------;;;

(in-package :booms-app)


;;;-------------------------------------------------------------------------;;;
;;;  M Y - P O P - U P - D I A L O G  functions                             ;;;
;;;-------------------------------------------------------------------------;;;
;;; This is my version of pop-up-dialog - it'll handle the window positioning
;;; and the events enabling.  There are also my private versions of standard
;;; dialogs for solving Allegro bugs that pop from time to time...

;;; The center argument should be :screen :cursor or nil.
(defun MY-POP-UP-DIALOG (dialog &optional center)
  (let ((pos #.(make-position 0 0))
        (no-events (not (events-enabled *booms-app*)))
        (*prevent-actions* t))
    (when center
      (case center
        ((:cursor) (ncopy-position pos (cursor-position *screen*)))
        ((:screen) (setf (position-x pos) (round (get-screen-size-x) 2))
                   (setf (position-y pos) (round (get-screen-size-y) 2))))
      (decf (position-x pos) (round (window-exterior-width  dialog) 2))
      (decf (position-y pos) (round (window-exterior-height dialog) 2))
      (setf (position-x pos)
            (min (max (position-x pos) 0)
                 (- (get-screen-size-x) (window-exterior-width dialog))))
      (setf (position-y pos)
            (min (max (position-y pos) 0)
                 (- (get-screen-size-y) (window-exterior-height dialog))))
      (move-window dialog pos))
    ;; Enable events temporarily if disabled, otherwise the Allegro will pop.
    (when no-events (enable-events *booms-app* t))
    (prog1 (pop-up-dialog dialog)
      (when no-events (enable-events *booms-app* nil)))))

;;; Macro to define my dialog function versions.
(defmacro DEFMYDIALOG (my-func original-func)
  `(defun ,my-func (&rest args)
     (let* ((*prevent-actions* t)
            (result (apply #',original-func args)))
       (select-window *booms-app*)
       result)))

;;; The alternative dialog functions.

(defmydialog MY-POP-UP-MESSAGE-DIALOG          pop-up-message-dialog)

(defmydialog MY-YES-OR-NO-P                    yes-or-no-p)

(defmydialog MY-ASK-USER-FOR-NEW-PATHNAME      ask-user-for-new-pathname)

(defmydialog MY-ASK-USER-FOR-EXISTING-PATHNAME ask-user-for-existing-pathname)


;;;-------------------------------------------------------------------------;;;
;;;  G E N E R A L  dialog variables                                        ;;;
;;;-------------------------------------------------------------------------;;;

;;; The dialog items, set on first use.
(defvar *BOOMS-LISP-DIALOG*    nil)
(defvar *BOOMS-STRING-DIALOG*  nil)
(defvar *BOOMS-LIST-DIALOG*    nil)
(defvar *BOOMS-WELCOME-DIALOG* nil)
(defvar *BOOMS-ABOUT-DIALOG*   nil)

;;; Resetting all dialogs.
(defun INITIALIZE-DIALOGS ()
  (setf *booms-lisp-dialog*    nil)
  (setf *booms-string-dialog*  nil)
  (setf *booms-list-dialog*    nil)
  (setf *booms-welcome-dialog* nil)
  (setf *booms-about-dialog*   nil))


;;;-------------------------------------------------------------------------;;;
;;;  D I A L O G - G E T - V A L U E  dialog box                            ;;;
;;;-------------------------------------------------------------------------;;;
;;; Used for reading a value from the user.  Uses *booms-lisp-dialog*.

(defun DIALOG-GET-VALUE (title default &optional cursor-center-p)
  (make-booms-lisp-dialog)
  (setf (stream-title *booms-lisp-dialog*) title)
  (setf (dialog-item-value (second (dialog-items *booms-lisp-dialog*)))
        default)
  (if (my-pop-up-dialog *booms-lisp-dialog*
                        (if cursor-center-p :cursor :screen))
    (dialog-item-value (second (dialog-items *booms-lisp-dialog*)))
    nil))

(defun MAKE-BOOMS-LISP-DIALOG ()
  (unless *booms-lisp-dialog*
    (setf *booms-lisp-dialog*
          (open-dialog
           (list
            (make-dialog-item
             :widget       'static-text
             :value        "Enter ~value:"
             :box          (make-box 20 20 100 42)
             :tabstop      nil
             :groupstart   t)
            (make-dialog-item
             :widget       'lisp-text
             :value        nil
             :box          (make-box 110 20 280 52)
             :font         (make-font :modern :courier\ new 24 '(:bold))
             :3d-border    t
             :tabstop      t
             :groupstart   nil)
            (make-dialog-item
             :widget       'default-button
             :title        "~OK"
             :box          (make-box 20 72 145 94)
             :3d-border    t
             :tabstop      t
             :groupstart   nil
             :set-value-fn 'return-t-from-pop-up-dialog)
            (make-dialog-item
             :widget       'cancel-button
             :title        "~Cancel"
             :box          (make-box 155 72 280 94)
             :3d-border    t
             :tabstop      t
             :groupstart   nil
             :set-value-fn 'return-nil-from-pop-up-dialog))
           'dialog *booms-app*
           :title nil
           :pop-up-p t
           :window-interior (make-box 0 0 300 114)))))


;;;-------------------------------------------------------------------------;;;
;;;  D I A L O G - G E T - S T R I N G  dialog box                          ;;;
;;;-------------------------------------------------------------------------;;;
;;; Used for reading a string from the user.  Uses *booms-lisp-dialog*.

(defun DIALOG-GET-STRING (title default &optional cursor-center-p)
  (make-booms-string-dialog)
  (setf (stream-title *booms-string-dialog*) title)
  (setf (dialog-item-value (second (dialog-items *booms-string-dialog*)))
        default)
  (if (my-pop-up-dialog *booms-string-dialog*
                        (if cursor-center-p :cursor :screen))
    (dialog-item-value (second (dialog-items *booms-string-dialog*)))
    nil))

(defun MAKE-BOOMS-STRING-DIALOG ()
  (unless *booms-string-dialog*
    (setf *booms-string-dialog*
          (open-dialog
           (list
            (make-dialog-item
             :widget       'static-text
             :value        "Enter ~string:"
             :box          (make-box 20 20 100 42)
             :tabstop      nil
             :groupstart   t)
            (make-dialog-item
             :widget       'editable-text
             :value        ""
             :box          (make-box 110 20 280 52)
             :font         (make-font :swiss :arial 24 '(:bold))
             :3d-border    t
             :tabstop      t
             :groupstart   nil)
            (make-dialog-item
             :widget       'default-button
             :title        "~OK"
             :box          (make-box 20 72 145 94)
             :3d-border    t
             :tabstop      t
             :groupstart   nil
             :set-value-fn 'return-t-from-pop-up-dialog)
            (make-dialog-item
             :widget       'cancel-button
             :title        "~Cancel"
             :box          (make-box 155 72 280 94)
             :3d-border    t
             :tabstop      t
             :groupstart   nil
             :set-value-fn 'return-nil-from-pop-up-dialog))
           'dialog *booms-app*
           :title nil
           :pop-up-p t
           :window-interior (make-box 0 0 300 114)))))


;;;-------------------------------------------------------------------------;;;
;;;  D I A L O G - G E T - L I S T - V A L U E  dialog box                  ;;;
;;;-------------------------------------------------------------------------;;;
;;; Used for reading a value from a list of choises.

(defun DIALOG-GET-LIST-VALUE (title options &optional cursor-center-p)
  (make-booms-list-dialog)
  (setf (stream-title *booms-list-dialog*) title)
  (let ((list-widget (second (dialog-items *booms-list-dialog*))))
    (set-dialog-item-range list-widget nil)
    (dolist (option options)
      (list-widget-add-item list-widget option))
    (unless (null options)
      (list-widget-set-index list-widget 0)))
  (resize-window *booms-list-dialog*
                 (make-position 280 (max 104
                                         (min (+ 86 (* 24 (length options)))
                                              300))))
  (if (my-pop-up-dialog *booms-list-dialog*
                        (if cursor-center-p :cursor :screen))
    (dialog-item-value (second (dialog-items *booms-list-dialog*)))
    nil))

(defun MAKE-BOOMS-LIST-DIALOG ()
  (unless *booms-list-dialog*
    (setf *booms-list-dialog*
          (open-dialog
           (list
            (make-dialog-item
             :widget            'static-text
             :value             "Select a ~value:"
             :box               (make-box 20 20 120 42)
             :tabstop           nil
             :groupstart        t
             :left-attachment   :left
             :right-attachment  :left
             :top-attachment    :top
             :bottom-attachment :top)
            (make-dialog-item
             :widget            'single-item-list
             :value             nil
             :key               'capitalize-object
             :box               (make-box 130 20 260 142)
             :3d-border         t
             :tabstop           t
             :groupstart        nil
             :double-click-fn #'(lambda (&rest args) (throw 'pop-up-dialog t))
             :font              (make-font :modern :courier\ new 24 '(:bold))
             :left-attachment   :left
             :right-attachment  :right
             :top-attachment    :top
             :bottom-attachment :bottom)
            (make-dialog-item
             :widget            'default-button
             :title             "~OK"
             :box               (make-box 20 162 135 184)
             :3d-border         t
             :tabstop           t
             :groupstart        nil
             :set-value-fn      'return-t-from-pop-up-dialog
             :left-attachment   :left
             :right-attachment  :left
             :top-attachment    :bottom
             :bottom-attachment :bottom)
            (make-dialog-item
             :widget            'cancel-button
             :title             "~Cancel"
             :box               (make-box 145 162 260 184)
             :3d-border         t
             :tabstop           t
             :groupstart        nil
             :set-value-fn      'return-nil-from-pop-up-dialog
             :left-attachment   :right
             :right-attachment  :right
             :top-attachment    :bottom
             :bottom-attachment :bottom))
           'dialog *booms-app*
           :title nil
           :pop-up-p t
           :window-interior (make-box 0 0 280 204)))))


;;;-------------------------------------------------------------------------;;;
;;;  W E L C O M E  dialog box & activation                                 ;;;
;;;-------------------------------------------------------------------------;;;

(defun POPUP-WELCOME-DIALOG ()
  (make-booms-welcome-dialog)
  (play-sound "intro.wav")
  (my-pop-up-dialog *booms-welcome-dialog* :screen))

(defun MAKE-BOOMS-WELCOME-DIALOG ()
  (unless *booms-welcome-dialog*
    (setf *booms-welcome-dialog*
          (open-dialog
           (list
            (make-dialog-item
             :widget           'cancel-button
             :title            "~Cancel"
             :box              (make-box 0 0 0 0)
             :hidden-p         t
             :window-state     :shrunk
             :set-value-fn     'return-nil-from-pop-up-dialog)
            (make-dialog-item
             :widget           'static-picture
             :value            "logo.bmp"
             :box              (make-box 20 20 580 380)
             :border           :plain
             :3d-border        t
             :tabstop          nil
             :groupstart       nil
             :background-color light-gray
             :stretching       nil)
            (make-dialog-item
             :widget           'default-button
             :title            "OK"
             :box              (make-box 25 25 575 375)
             :hidden-p         t
             :set-value-fn     'return-t-from-pop-up-dialog))
           'dialog *booms-app*
           :title             "Welcome to BOOMS!"
           :window-state      nil
           :window-border     :dialog-box
           :left-attachment   nil
           :top-attachment    nil
           :right-attachment  nil
           :bottom-attachment nil
           :user-movable      nil
           :user-resizable    nil
           :user-closable     nil
           :user-shrinkable   nil
           :user-scrollable   nil
           :overlapped        nil
           :background-color  light-gray
           :pop-up-p          t
           :window-interior   (make-box 0 0 600 400)))))


;;;-------------------------------------------------------------------------;;;
;;;  A B O U T  dialog box construction                                     ;;;
;;;-------------------------------------------------------------------------;;;
;;; It is a function so it will be bound to the booms window and use the
;;; *booms-text* that will be ready only after loading.

(defun POPUP-ABOUT-DIALOG ()
  (make-booms-about-dialog)
  (my-pop-up-dialog *booms-about-dialog* :screen))

(defun MAKE-BOOMS-ABOUT-DIALOG ()
  (unless *booms-about-dialog*
    (setf *booms-about-dialog*
          (open-dialog
           (list
            (make-dialog-item
             :widget 'default-button
             :title "~OK"
             :box (make-box (- (get-screen-size-x) 130)
                            (- (get-screen-size-y) 60)
                            (- (get-screen-size-x) 30)
                            (- (get-screen-size-y) 30))
             :3d-border    t
             :tabstop      nil
             :groupstart   nil
             :set-value-fn 'return-t-from-pop-up-dialog)
            (make-dialog-item
             :widget 'static-text
             :value "BOOMS!"
             :box (make-box (- (/ (- (get-screen-size-x) 20) 2) 160)
                            (- (get-screen-size-y) 75)
                            (- (/ (- (get-screen-size-x) 20) 2) 10)
                            (- (get-screen-size-y) 30))
             :border           :plain
             :3d-border        t
             :tabstop          nil
             :groupstart       t
             :background-color light-gray
             :justification    :center
             :3d-text          t
             :font             (make-font :swiss :arial 37 '(:bold :italic)))
            (make-dialog-item
             :widget 'picture-button
             :title         (booms-icon 5)
             :box (make-box (+ (/ (- (get-screen-size-x) 20) 2) 10)
                            (- (get-screen-size-y) 75)
                            (+ (/ (- (get-screen-size-x) 20) 2) 160)
                            (- (get-screen-size-y) 30))
             :border       :plain
             :3d-border    t
             :tabstop      t
             :stretching   t
             :set-value-fn #'(lambda (item &rest args)
                               (popup-welcome-dialog) (values nil t)))
            (make-dialog-item
             :widget 'multi-line-editable-text
             :value cl-user::*booms-text*
             :box (make-box 30 30 (- (get-screen-size-x) 45)
                                  (- (get-screen-size-y) 90))
             :border      :plain
             :3d-border   t
             :tabstop     t
             :groupstart  t
             :read-only   t
             :3d-text     t
             :font        (make-font :modern :courier\ new 21 '(:bold))))
           'dialog *booms-app*
           :window-state      nil
           :window-border     :dialog-box
           :left-attachment   nil
           :top-attachment    nil
           :right-attachment  nil
           :bottom-attachment nil
           :user-movable      nil
           :user-resizable    nil
           :user-closable     nil
           :user-shrinkable   nil
           :user-scrollable   nil
           :overlapped        nil
           :background-color  light-gray
           :pop-up-p          t
           :window-interior   (make-box 0 0
                                        (- (get-screen-size-x) 10)
                                        (- (get-screen-size-y) 10))))))


;;;-------------------------------------------------------------------------;;;
;;;  D O N E !                                                              ;;;
;;;-------------------------------------------------------------------------;;;

(provide :DIALOGS)

