;;;                   -*- Mode: Lisp; Package: GCALC -*-                    ;;;
;;; ----------------------------------------------------------------------- ;;;
;;; File:         gcalc.lsp                                                 ;;;
;;; Description:  BOOMS gcalc documents                                     ;;;
;;; Author:       Eli Barzilay - eli@cs.bgu.ac.il                           ;;;
;;; Date:         29-12-96                                                  ;;;
;;; Package:      GCALC                                                     ;;;
;;; ----------------------------------------------------------------------- ;;;

#|

This file is part of the BOOMS system:
+-----------------------------------------------------------------------------+
|                                                                             |
| /######   /#####   /#####  /##     /##  /#####                              |
| /##///## /##///## /##///## /###   /### /##///##                        /--- |
| /##  /## /##  /## /##  /## /#### /#### /##  ///          /------/     /     |
| /######/ /##  /## /##  /## /##/####/## //#####          /      /     /      |
| /##///## /##  /## /##  /## /##//##//##  /////##        /------/ __  /       |
| /##  /## /##  /## /##  /## /## /// /##      /##   __  /  __  / /  \/        |
| /##  /## /##  /## /##  /## /##     /## /##  /##  /  \/  /  \/  \__/         |
| /######/ //#####/ //#####/ /##     /## //#####/  \__/   \__/                |
| ///////   //////   //////  ///     ///  //////                              |
|                                                                             |
|                   Booms Object Oriented Music Structures.                   |
|Eli Barzilay:                                                   Maze is Life!|
+-----------------------------------------------------------------------------+

|#


;;;-------------------------------------------------------------------------;;;
;;;  G C A L C  file                                                        ;;;
;;;-------------------------------------------------------------------------;;;

(in-package :gcalc)


;;;-------------------------------------------------------------------------;;;
;;;  G C A L C  variables                                                   ;;;
;;;-------------------------------------------------------------------------;;;

;;; The popping palette with the expression drawings.
(defvar *GCALC-DRAW-WINDOW* nil)

;;; The GCalc expression icon to draw.
(defvar *GCALC-DRAW-ICON*   nil)


;;;-------------------------------------------------------------------------;;;
;;;  G C A L C  expressions                                                 ;;;
;;;-------------------------------------------------------------------------;;;

;;; The base class for all GCalc values.
(defclass GCALC-EXP (savable) ())

;;; Simple expressions has only a value - an RGB color or :none.
(defclass SIMPLE-EXP (gcalc-exp)
  ((val :initarg :val :reader val)))

(defmethod LOAD-SLOTS append ((x simple-exp))
  '(val))

;;; The variable expression - had one number slot that make different variables
;;; of the same color distinguishable.  This number is not needed as Lisp "eq"
;;; test is used.
(defclass VAR-EXP (simple-exp) ())

;;; Simple expressions has only a value - an RGB color or :none.
(defclass COMPOUND-EXP (gcalc-exp)
  ((val1 :initarg :val1 :reader val1)
   (val2 :initarg :val2 :reader val2)))

(defmethod LOAD-SLOTS append ((x compound-exp))
  '(val1 val2))

;;; Definitions of the compound expression types.

(defclass LEFT-RIGHT-EXP (compound-exp) ())

(defclass TOP-BOTTOM-EXP (compound-exp) ())

(defclass FRONT-BACK-EXP (compound-exp) ())

(defclass APPLICATION-EXP (compound-exp) ())

(defclass ABSTRACTION-EXP (compound-exp) ())


;;;-------------------------------------------------------------------------;;;
;;;  G C A L C  window classes declaration                                  ;;;
;;;-------------------------------------------------------------------------;;;

(defclass GCALC-FRAME (doc-frame) ())

(defclass GCALC-DOC (document) ())


;;;-------------------------------------------------------------------------;;;
;;;  G C A L C - I C O N  general superclass                                ;;;
;;;-------------------------------------------------------------------------;;;

;;; This is the base class for all GCalc nodes.
(defclass GCALC-ICON () ())

;;; Evaluating a GCalc icon means drawing its value.
(defmethod ICON-EVAL ((icon gcalc-icon))
  (when (call-next-method)
    (let ((*gcalc-draw-icon* icon))
      (my-pop-up-dialog *gcalc-draw-window* :cursor))))

;;; This will be the base-class of GCalc icons that have a memoized bitmap
;;; attached - in fact, all icons except the gcalc-identity icons.
(defclass GCALC-BITMAP-ICON (gcalc-icon)
  ((saved-bitmap :accessor saved-bitmap :initform nil)
   (bitmap-ok-p  :accessor bitmap-ok-p  :initform nil)))

;;; Forget also the saved bitmap.
(defmethod FORGET-VALUE :after ((obj gcalc-bitmap-icon))
  (setf (bitmap-ok-p obj) nil))


;;;-------------------------------------------------------------------------;;;
;;;  G C A L C  identity icon                                               ;;;
;;;-------------------------------------------------------------------------;;;

;;; The GCalc identity icon.
(defclass GCALC-IDENTITY (gcalc-icon iconic-identity)
  ((saved-bitmap :allocation :class)
   (bitmap-ok-p  :allocation :class)))

(set-booms-name gcalc-identity 'identity)

;;; Return this class for the :gcalc doc-type.
(defmethod ICONIC-IDENTITY-CLASS-NAME ((doctype (eql :gcalc))) 'gcalc-identity)

;;; Make gcalc-identity use its argument's bitmap.  No checking of existing
;;; argument needed since it'll be used only after a successful evaluation.

(defmethod SAVED-BITMAP ((icon gcalc-identity))
  (saved-bitmap (getarg0 icon 0)))

(defmethod (setf SAVED-BITMAP) (newval (icon gcalc-identity))
  (setf (saved-bitmap (getarg0 icon 0)) newval))

(defmethod BITMAP-OK-P ((icon gcalc-identity))
  (bitmap-ok-p (getarg0 icon 0)))

(defmethod (setf BITMAP-OK-P) (newval (icon gcalc-identity))
  (setf (bitmap-ok-p (getarg0 icon 0)) newval))


;;;-------------------------------------------------------------------------;;;
;;;  G C A L C  abstractions                                                ;;;
;;;-------------------------------------------------------------------------;;;

;;; Tha GCalc specific iconic-abstraction classes.

(defclass GCALC-ABSTRACT
  (gcalc-bitmap-icon iconic-abstract) ())

(set-booms-name gcalc-abstract 'abstract)

(defclass GCALC-ABSTRACT-LEAF
  (gcalc-bitmap-icon iconic-abstract-leaf) ())

(set-booms-name gcalc-abstract-leaf 'abstract-leaf)

(defclass GCALC-NAMED-ABSTRACT
  (gcalc-bitmap-icon iconic-named-abstract) ())

(defclass GCALC-NAMED-ABSTRACT-LEAF
  (gcalc-bitmap-icon iconic-named-abstract-leaf) ())

;;; This are the specialization method versions for the GCalc abstraction
;;; classes.

(defmethod ICONIC-ABSTRACT-CLASS-NAME ((doctype (eql :gcalc)))
  'gcalc-abstract)

(defmethod ICONIC-ABSTRACT-LEAF-CLASS-NAME ((doctype (eql :gcalc)))
  'gcalc-abstract-leaf)

(defmethod ICON-NAMED-ABSTRACTION-CLASS ((doctype (eql :gcalc))
                                         (icon gcalc-abstract))
  'gcalc-named-abstract)

(defmethod ICON-NAMED-ABSTRACTION-CLASS ((doctype (eql :gcalc))
                                         (icon gcalc-abstract-leaf))
  'gcalc-named-abstract-leaf)

(defmethod ICON-UNNAMED-ABSTRACTION-CLASS ((doctype (eql :gcalc))
                                           (icon gcalc-named-abstract))
  'gcalc-abstract)

(defmethod ICON-UNNAMED-ABSTRACTION-CLASS ((doctype (eql :gcalc))
                                           (icon gcalc-named-abstract-leaf))
  'gcalc-abstract-leaf)


;;;-------------------------------------------------------------------------;;;
;;;  G C A L C  binary functions                                            ;;;
;;;-------------------------------------------------------------------------;;;
;;; These are the GCalc functions - not the constructors.  Evaluating an
;;; expression to produce a result gcalc-exp value is done by the evaluate
;;; method.

;;; This is the base class for all of GCalc's simple binary functions -
;;; producing immidiate constructs.
(deficon BINARY-FUNC ((1st gcalc-exp) (2nd gcalc-exp)) (gcalc-bitmap-icon)
  ((exp-type :reader exp-type))
  (evaluate gcalc-exp := (make-instance (exp-type self) :val1 1st :val2 2nd)))

;;; This is the reversed versions of the above.
(deficon BINARY-REV-FUNC ((1st gcalc-exp) (2nd gcalc-exp)) (binary-func) ()
  (evaluate gcalc-exp := (make-instance (exp-type self) :val1 2nd :val2 1st)))

;;; These are the funcs.

(defclass LEFT-RIGHT (binary-func)
  ((exp-type :allocation :class :initform 'left-right-exp)))

(defclass TOP-BOTTOM (binary-func)
  ((exp-type :allocation :class :initform 'top-bottom-exp)))

(defclass FRONT-BACK (binary-func)
  ((exp-type :allocation :class :initform 'front-back-exp)))

(defclass RIGHT-LEFT (binary-rev-func left-right) ())

(defclass BOTTOM-TOP (binary-rev-func top-bottom) ())

(defclass BACK-FRONT (binary-rev-func front-back) ())

;;; Associating these operators with the correct icons.

(defmethod ICON-ICON ((icon left-right)) (booms-icon 27))

(defmethod ICON-ICON ((icon top-bottom)) (booms-icon 28))

(defmethod ICON-ICON ((icon front-back)) (booms-icon 29))

(defmethod ICON-ICON ((icon right-left)) (booms-icon 30))

(defmethod ICON-ICON ((icon bottom-top)) (booms-icon 31))

(defmethod ICON-ICON ((icon back-front)) (booms-icon 32))

;;; GCalc lambda function, producing abstraction values (GCalc abstractions).

(deficon LAMBDA-FUNC ((var simple-exp) (exp gcalc-exp)) (gcalc-bitmap-icon) ()
  (evaluate abstraction-exp :=
    (let ((var-exp (make-instance 'var-exp :val (val var))))
      (make-instance 'abstraction-exp
                     :val1 var-exp
                     :val2 (replace-simple-exp exp var var-exp)))))

(set-booms-name lambda-func 'lambda)

(defmethod ICON-ICON ((icon lambda-func)) (booms-icon 21))

;;; GCalc apply function.

(deficon APPLY-FUNC ((exp1 gcalc-exp) (exp2 gcalc-exp)) (gcalc-bitmap-icon) ()
  (evaluate gcalc-exp :=
    (eval-exp (make-instance 'application-exp :val1 exp1 :val2 exp2))))

(set-booms-name apply-func 'apply)

(defmethod ICON-ICON ((icon apply-func)) (booms-icon 22))


;;;-------------------------------------------------------------------------;;;
;;;  G C A L C  evaluation function                                         ;;;
;;;-------------------------------------------------------------------------;;;

;;; Evaluation of a GCalc expression - reducing applications.
(defun EVAL-EXP (exp)
  (typecase exp
    ((or simple-exp abstraction-exp) exp)
    (application-exp
     (let ((1st (eval-exp (val1 exp))) (2nd (val2 exp)))
       (typecase 1st
         ((or simple-exp application-exp) exp)
         (abstraction-exp
          (eval-exp (replace-simple-exp (val2 1st) (val1 1st) 2nd)))
         (t

          (let ((2nd (split-exp (eval-exp 2nd) (type-of 1st))))
            (make-instance
             (type-of 1st)
             :val1 (eval-exp
                    (make-instance 'application-exp
                                   :val1 (val1 1st)
                                   :val2 (val1 2nd)))
             :val2 (eval-exp
                    (make-instance 'application-exp
                                   :val1 (val2 1st)
                                   :val2 (val2 2nd)))))))))
    (t
     (make-instance (type-of exp)
                    :val1 (eval-exp (val1 exp))
                    :val2 (eval-exp (val2 exp))))))

;; Split an expression - used for applying constructions of functions.
(defun SPLIT-EXP (exp op)
  (cond
   ((typep exp '(or simple-exp abstraction-exp application-exp))
    (make-instance op :val1 exp :val2 exp))
   ((eq (type-of exp) op) exp)
   (t (let* ((e1   (split-exp (val1 exp) op))
             (e2   (split-exp (val2 exp) op))
             (e11  (val1 e1))
             (e12  (val2 e1))
             (e21  (val1 e2))
             (e22  (val2 e2))
             (e-op (type-of exp)))
        (make-instance op
                       :val1 (make-instance e-op :val1 e11 :val2 e21)
                       :val2 (make-instance e-op :val1 e12 :val2 e22))))))


;;;-------------------------------------------------------------------------;;;
;;;  G C A L C  utility functions                                           ;;;
;;;-------------------------------------------------------------------------;;;

;;; Are two simple expressions equal?
(defun EQUAL-SIMPLE-EXP (exp1 exp2)
  (cond
    ((and (typep exp1 'var-exp) (typep exp2 'var-exp))
     (eq exp1 exp2))
    ((or (typep exp1 'var-exp) (typep exp2 'var-exp))
     nil)
    ((and (typep exp1 'simple-exp) (typep exp2 'simple-exp))
     (equalp (val exp1) (val exp2)))
    (t nil)))

;;; Return exp with var replaced by val.  from-exp can be any simple-exp -
;;; variable or color value.  This is done efficiently by replace-simple-exp1,
;;; which returns nil in case of no substitution needed or the substituted
;;; expression, so only subtrees that contain from-exp are copied.
(defun REPLACE-SIMPLE-EXP (exp from-exp to-exp)
  (labels ((replace-simple-exp1 (exp)
             (typecase exp
               (simple-exp
                (if (equal-simple-exp exp from-exp) to-exp exp))
               (compound-exp
                (if (and (typep exp 'abstraction-exp)
                         (equal-simple-exp (val1 exp) from-exp))
                  exp
                  (let ((replaced1 (replace-simple-exp1 (val1 exp)))
                        (replaced2 (replace-simple-exp1 (val2 exp))))
                    (make-instance (type-of exp)
                                   :val1 replaced1 :val2 replaced2)))))))
    (replace-simple-exp1 exp)))


;;;-------------------------------------------------------------------------;;;
;;;  G C A L C  leaf objects                                                ;;;
;;;-------------------------------------------------------------------------;;;

;;; The base class for leaves - colors.
(defleaf BASIC-COLOR (gcalc-bitmap-icon)
  ((type-out :allocation :class :initform 'simple-exp)))

(set-booms-name basic-color 'g-color)

;;; Ask user for initial value using the g-color pane.
(defmethod LEAF-INIT-VAL/PANE ((icon basic-color))
  (values (make-instance 'simple-exp :val black) :g-color))

;;; Describing an iconic-leaf with a string.
(defmethod ICON-DESCRIBE ((icon basic-color) &optional socket)
  (format nil "~:(~a~)   ( ~a )"
          (icon-describe-name icon)
          (arguments icon)))

;;; Drawing an icon for color leaves - draw the color.
(defmethod ICON-DRAW-ICON ((icon basic-color))
  (let* ((pos   #.(make-position 0 0))
         (doc   (icon-doc icon))
         (color (val (arguments icon)))
         (rgb-p (rgb-p color)))
    (nmake-position pos #1= #.(+ +icon-margin+ (round +win-icon-size+ 2)) #1#)
    (nposition+ pos (icon-pos icon))
    (with-foreground-color (doc (if rgb-p color white))
      (fill-circle doc pos #.(round +win-icon-size+ 2)))
    (with-line-width (doc #.(round +border-line-width+ 2))
      (with-foreground-color (doc (icon-fg-color icon))
        (draw-circle doc pos #.(round +win-icon-size+ 2)))
      (unless rgb-p
        (with-foreground-color (doc black)
          (nposition- pos #.(make-position (round +win-icon-size+ 2)
                                           (round +win-icon-size+ 2)))
          (let* ((x1 (position-x pos))
                 (y1 (position-y pos))
                 (x2 (+ x1 +win-icon-size+))
                 (y2 (+ y1 +win-icon-size+)))
            (draw-line doc
                       (nmake-position #.(make-position 0 0) x1 y1)
                       (nmake-position #.(make-position 0 0) x2 y2))
            (draw-line doc
                       (nmake-position #.(make-position 0 0) x1 y2)
                       (nmake-position #.(make-position 0 0) x2 y1))))))))


;;;-------------------------------------------------------------------------;;;
;;;  G C A L C  leaf constants                                              ;;;
;;;-------------------------------------------------------------------------;;;

;;; Constant color icons.
(defconstleaf CONSTANT-COLOR (basic-color) ())

;;; Macro for a definition of a constant numeral leaf.
(defmacro DEFCONSTCOLOR (sym name redval greenval blueval)
  `(progn
     (defleaf ,sym (constant-color)
       ((arguments :initform
                   (make-instance
                    'simple-exp :val
                    (make-rgb :red ,redval :green ,greenval :blue ,blueval)))
        ;; Have shared bitmap/flag for all constant of the same class.
        (saved-bitmap :allocation :class)
        (bitmap-ok-p  :allocation :class)))
     (set-booms-name ,sym ,name)))

;;; These are some color constants.

(defconstcolor BLACK-CONST      'BLACK      #x00 #x00 #x00)

(defconstcolor GRAY-CONST       'GRAY       #x80 #x80 #x80)

(defconstcolor LT-GRAY-CONST    'LT-GRAY    #xC0 #xC0 #xC0)

(defconstcolor WHITE-CONST      'WHITE      #xFF #xFF #xFF)

(defconstcolor RED-CONST        'RED        #xFF #x00 #x00)

(defconstcolor GREEN-CONST      'GREEN      #x00 #xFF #x00)

(defconstcolor BLUE-CONST       'BLUE       #x00 #x00 #xFF)

(defconstcolor YELLOW-CONST     'YELLOW     #xFF #xFF #x00)

(defconstcolor CYAN-CONST       'CYAN       #x00 #xFF #xFF)

(defconstcolor MAGENTA-CONST    'MAGENTA    #xFF #x00 #xFF)

(defconstcolor DK-RED-CONST     'DK-RED     #x80 #x00 #x00)

(defconstcolor DK-GREEN-CONST   'DK-GREEN   #x00 #x80 #x00)

(defconstcolor DK-BLUE-CONST    'DK-BLUE    #x00 #x00 #x80)

(defconstcolor DK-YELLOW-CONST  'DK-YELLOW  #x80 #x80 #x00)

(defconstcolor DK-CYAN-CONST    'DK-CYAN    #x00 #x80 #x80)

(defconstcolor DK-MAGENTA-CONST 'DK-MAGENTA #x80 #x00 #x80)

;;; This represents a special transparent color.
(defleaf TRANSPARENT-CONST (constant-color)
  ((arguments :initform (make-instance 'simple-exp :val :none))))

(set-booms-name transparent-const 'transparent)


;;;-------------------------------------------------------------------------;;;
;;;  G - C O L O R  leaf edit page                                          ;;;
;;;-------------------------------------------------------------------------;;;

;;; The basic-color - g-color widgets.
(defvar *ICON-EDIT-G-COLOR-COLOR-BOX*   nil)
(defvar *ICON-EDIT-G-COLOR-COLOR-RED*   nil)
(defvar *ICON-EDIT-G-COLOR-COLOR-GREEN* nil)
(defvar *ICON-EDIT-G-COLOR-COLOR-BLUE*  nil)

;;; The basic-color edit page.
(defun MAKE-BASIC-COLOR-TAB ()
  (add-icon-edit-tab :g-color)
  (progrev
   ;; The color widget.
   (add-icon-edit-tab-widget
     :widget          'static-text
     :value           "Basic ~Color:"
     :box             (make-box 10 30 110 55)
     :tabstop         nil
     :groupstart      t)
   (setf *icon-edit-g-color-color-box*
    (add-icon-edit-tab-widget
     :widget         'color-box
     :value          nil
     :box            (make-box 10 60 70 85)
     :tabstop        nil
     :groupstart     nil
     :set-value-fn   #'color-box-g-color-set))
   (setf *icon-edit-g-color-color-red*
    (add-icon-edit-tab-widget
     :widget          'lisp-text
     :name            :red-g-color-color-text
     :value           nil
     :box             (make-box 77 60 112 85)
     :tabstop         t
     :groupstart      nil
     :up-down-control :red-g-color-color-up-down
     :set-value-fn    #'color-g-color-red-set))
   (add-icon-edit-tab-widget
     :widget          'up-down-control
     :name            :red-g-color-color-up-down
     :value           nil
     :box             (make-box 112 60 122 85)
     :tabstop         nil
     :groupstart      nil
     :buddy-widget    :red-g-color-color-text
     :buddy-side      :right
     :delayed         nil
     :orientation     :vertical
     :wrapping        nil
     :use-arrow-keys  t
     :increment       nil
     :range           (list 0 255))
   (setf *icon-edit-g-color-color-green*
    (add-icon-edit-tab-widget
     :widget          'lisp-text
     :name            :green-g-color-color-text
     :value           nil
     :box             (make-box 131 60 166 85)
     :tabstop         t
     :groupstart      nil
     :up-down-control :green-g-color-color-up-down
     :set-value-fn    #'color-g-color-green-set))
   (add-icon-edit-tab-widget
     :widget          'up-down-control
     :name            :green-g-color-color-up-down
     :value           nil
     :box             (make-box 166 60 176 85)
     :tabstop         nil
     :groupstart      nil
     :buddy-widget    :green-g-color-color-text
     :buddy-side      :right
     :delayed         nil
     :orientation     :vertical
     :wrapping        nil
     :use-arrow-keys  t
     :increment       nil
     :range           (list 0 255))
   (setf *icon-edit-g-color-color-blue*
    (add-icon-edit-tab-widget
     :widget          'lisp-text
     :name            :blue-g-color-color-text
     :value           nil
     :box             (make-box 185 60 220 85)
     :tabstop         t
     :groupstart      nil
     :up-down-control :blue-g-color-color-up-down
     :set-value-fn    #'color-g-color-blue-set))
   (add-icon-edit-tab-widget
     :widget          'up-down-control
     :name            :blue-g-color-color-up-down
     :value           nil
     :box             (make-box 220 60 230 85)
     :tabstop         nil
     :groupstart      nil
     :buddy-widget    :blue-g-color-color-text
     :buddy-side      :right
     :delayed         nil
     :orientation     :vertical
     :wrapping        nil
     :use-arrow-keys  t
     :increment       nil
     :range           (list 0 255))
   (add-icon-edit-tab-widget
     :widget       'button
     :title        "~Transparent"
     :box          (make-box 77 90 230 115)
     :3d-border    t
     :tabstop      t
     :groupstart   nil
     :set-value-fn #'(lambda (&rest r)
                       (set-dialog-item-value
                        *icon-edit-g-color-color-box* :none)))
   ;; The apply button.
   (add-icon-edit-tab-widget
     :widget       'button
     :title        "~A p p l y"
     :box          (make-box 20 125 220 145)
     :3d-border    t
     :tabstop      t
     :groupstart   nil
     :set-value-fn #'(lambda (&rest r)
                       (let ((newcolor
                              (dialog-item-value
                               *icon-edit-g-color-color-box*)))
                         (setf (arguments *icon-edit-icon*)
                               (make-instance 'simple-exp :val newcolor))
                         (do-redraws (icon-doc *icon-edit-icon*))
                         nil)))))

;;; The color item handlers.

(defun COLOR-BOX-G-COLOR-SET (item new old)
  (unless *called-programmatically*
    (when (rgb-p new)
      (let ((*called-programmatically* t))
        (set-dialog-item-value
         *icon-edit-g-color-color-red*   (rgb-red   new))
        (set-dialog-item-value
         *icon-edit-g-color-color-green* (rgb-green new))
        (set-dialog-item-value
         *icon-edit-g-color-color-blue*  (rgb-blue  new)))))
  t)

(defun COLOR-G-COLOR-RED-SET (item new old)
  (cond (*called-programmatically* t)
        ((and (integerp new) (<= 0 new 255))
         (color-update *icon-edit-g-color-color-box*
                       new
                       (dialog-item-value *icon-edit-g-color-color-green*)
                       (dialog-item-value *icon-edit-g-color-color-blue*))
         t)
        (t nil)))

(defun COLOR-G-COLOR-GREEN-SET (item new old)
  (cond (*called-programmatically* t)
        ((and (integerp new) (<= 0 new 255))
         (color-update *icon-edit-g-color-color-box*
                       (dialog-item-value *icon-edit-g-color-color-red*)
                       new
                       (dialog-item-value *icon-edit-g-color-color-blue*))
         t)
        (t nil)))

(defun COLOR-G-COLOR-BLUE-SET (item new old)
  (cond (*called-programmatically* t)
        ((and (integerp new) (<= 0 new 255))
         (color-update *icon-edit-g-color-color-box*
                       (dialog-item-value *icon-edit-g-color-color-red*)
                       (dialog-item-value *icon-edit-g-color-color-green*)
                       new)
         t)
        (t nil)))

;;; Initialize the page values.
(defmethod ICON-EDIT-SETUP-PAGE ((icon basic-color) (id (eql :g-color))
                                 &optional socket)
  (set-dialog-item-value *icon-edit-g-color-color-box*
                         (val (arguments icon))))

;;; Terminate the page.
(defmethod ICON-EDIT-DONE-PAGE ((icon basic-color) (id (eql :g-color)) result)
  (when result
    (setf (arguments icon)
          (make-instance
           'simple-exp
           :val (dialog-item-value *icon-edit-g-color-color-box*)))
    (do-redraws (icon-doc icon))))

;;; Return only this page and the colors page for basic-color object.
(defmethod ICON-TAB-IDS ((x basic-color))
  (list :g-color :colors))


;;;-------------------------------------------------------------------------;;;
;;;  G C A L C  expression drawing                                          ;;;
;;;-------------------------------------------------------------------------;;;

;;; Some constants.
(defconstant +3D-DX+          0.4)
(defconstant +3D-DY+          0.25)
(defconstant +3D-BASIC-SIZE+  300)
(defconstant +3D-SIZE-X+      (round (* +3d-basic-size+ (1+ +3d-dx+))))
(defconstant +3D-SIZE-Y+      (round (* +3d-basic-size+ (1+ +3d-dy+))))
(defconstant +3D-MARGIN+      15)
(defconstant +3D-FULL-SIZE-X+ (+ +3d-size-x+ (* 2 +3d-margin+)))
(defconstant +3D-FULL-SIZE-Y+ (+ +3d-size-y+ (* 2 +3d-margin+)))
(defconstant +3D-LINE-WIDTH+  2)
(defconstant +ABS-GAP+        0.1)
(defconstant +ABS-RADIUS+     0.15)

;;; Drawing the widget's expression.
(defun DRAW-EXP (win exp x1 y1 z1 x2 y2 z2 abs-level)
  (macrolet ((x-to-scr (x)
               ;; This converts a number in the range [0,1 + 3d-dx] to sceen X
               ;; coordinates.
               `(round
                 (+ (* ,x
                       #.(/ (1+ +3d-dx+)) ; Consider total float size (1+dx).
                       +3d-size-x+)       ; Translate to window size.
                    +3d-margin+)))        ; Add margin.
             (y-to-scr (y)
               ;; This converts a number in the range [0,1 + 3d-dy] to sceen Y
               ;; coordinates.
               `(round
                 (+ (* ,y
                       #.(/ (1+ +3d-dy+)) ; Consider total float size (1+dy).
                       +3d-size-y+)       ; Translate to window size.
                    +3d-margin+))))       ; Add margin.
    (typecase exp
      (simple-exp
       (when (rgb-p (val exp))
         (let* (;;
                ;; Calculate points:
                ;;
                ;;       bx0 bx1    bx2 bx3
                ;;        |   |      |   |
                ;;  by0---P1---------P2  .
                ;;        |\  .       \  .
                ;;        | \ .        \ .
                ;;        |  \.         \.
                ;;  by1---|...P0---------P3
                ;;        |   |          |
                ;;  by2---P6..|..........|
                ;;         \  |          |
                ;;          \ |          |
                ;;           \|          |
                ;;  by3---....P5---------P4
                ;;
                ;;  (bx1 - x1) = (x2 - bx2) = (+3d-dx+ * (x2 - x1))
                ;;    and the same for y values.
                ;;
                (dx    (* +3d-dx+ (- z2 z1)))
                (bx0   (+ x1 (* z1 +3d-dx+)))
                (bx1   (+ bx0 dx))
                (bx2   (+ x2 (* z1 +3d-dx+)))
                (bx3   (+ bx2 dx))
                (dy    (* +3d-dy+ (- z2 z1)))
                (by0   (+ y1 (* z1 +3d-dy+)))
                (by1   (+ by0 dy))
                (by2   (+ y2 (* z1 +3d-dy+)))
                (by3   (+ by2 dy))
                (x-0   (x-to-scr bx0))
                (x-1   (x-to-scr bx1))
                (x-2   (x-to-scr bx2))
                (x-3   (x-to-scr bx3))
                (y-0   (y-to-scr by0))
                (y-1   (y-to-scr by1))
                (y-2   (y-to-scr by2))
                (y-3   (y-to-scr by3))
                (P     (nmake-position #0= #.(make-position 0 0) x-1 y-1))
                (P     (nmake-position #1= #.(make-position 0 0) x-0 y-0))
                (P     (nmake-position #2= #.(make-position 0 0) x-2 y-0))
                (P     (nmake-position #3= #.(make-position 0 0) x-3 y-1))
                (P     (nmake-position #4= #.(make-position 0 0) x-3 y-3))
                (P     (nmake-position #5= #.(make-position 0 0) x-1 y-3))
                (P     (nmake-position #6= #.(make-position 0 0) x-0 y-2))
                (poly1 '(#0# #1# #2# #3#))
                (poly2 '(#0# #3# #4# #5#))
                (poly3 '(#0# #5# #6# #1#)))
           (with-line-width (win +3d-line-width+)
             (with-foreground-color (win (val exp))
               (fill-polygon win poly1)
               (fill-polygon win poly2)
               (fill-polygon win poly3))
             (with-foreground-color (win (if (equalp (val exp) black)
                                           white black))
               (draw-polygon win poly1)
               (draw-polygon win poly2)
               (draw-polygon win poly3))))))
      (left-right-exp
       (let ((x12 (/ (+ x1 x2) 2)))
         (draw-exp win (val2 exp) x12 y1 z1 x2 y2 z2 0.0)
         (draw-exp win (val1 exp) x1 y1 z1 x12 y2 z2 0.0)))
      (top-bottom-exp
       (let ((y12 (/ (+ y1 y2) 2)))
         (draw-exp win (val2 exp) x1 y12 z1 x2 y2 z2 0.0)
         (draw-exp win (val1 exp) x1 y1 z1 x2 y12 z2 0.0)))
      (front-back-exp
       (let ((z12 (/ (+ z1 z2) 2)))
         (draw-exp win (val2 exp) x1 y1 z1 x2 y2 z12 0.0)
         (draw-exp win (val1 exp) x1 y1 z12 x2 y2 z2 0.0)))
      (abstraction-exp
       (draw-exp win (val2 exp) x1 y1 z1 x2 y2 z2 (+ +abs-gap+ abs-level))
       (let ((x-1   (x-to-scr (+ x1 (* (- x2 x1) (+ abs-level +abs-radius+))
                                 (* +3d-dx+ z2))))
             (y-1   (y-to-scr (+ y1 (* (- y2 y1) +abs-radius+)
                                 (* +3d-dy+ z2))))
             (x-r   (- (x-to-scr (* (- x2 x1) +abs-radius+)) +3d-margin+))
             (y-r   (- (y-to-scr (* (- y2 y1) +abs-radius+)) +3d-margin+))
             (color (val (val1 exp))))
         (with-line-width (win +3d-line-width+)
           (when (rgb-p color)
             (with-foreground-color (win color)
               (fill-ellipse win
                             (nmake-position #.(make-position 0 0) x-1 y-1)
                             x-r y-r 0)))
           (with-foreground-color (win (if (equalp color black) white black))
             (draw-ellipse win
                           (nmake-position #.(make-position 0 0) x-1 y-1)
                           x-r y-r 0)
             (unless (rgb-p color)
               (let ((dx (round (* x-r #.(cos (/ pi 4)))))
                     (dy (round (* y-r #.(sin (/ pi 4))))))
                 (draw-line win
                            (nmake-position #.(make-position 0 0)
                                            (- x-1 dx) (- y-1 dy))
                            (nmake-position #.(make-position 0 0)
                                            (+ x-1 dx) (+ y-1 dy)))
                 (draw-line win
                            (nmake-position #.(make-position 0 0)
                                            (- x-1 dx) (+ y-1 dy))
                            (nmake-position #.(make-position 0 0)
                                            (+ x-1 dx) (- y-1 dy))))))))))))


;;;-------------------------------------------------------------------------;;;
;;;  G C A L C  drawing palette                                             ;;;
;;;-------------------------------------------------------------------------;;;

;;; The drawing palette classes.

(defclass GCALC-DRAWING-WINDOW (bitmap-window)
  ((gcalc-window-done :accessor gcalc-window-done :initform nil)
   (last-icon         :accessor last-icon         :initform nil)))

(defclass GCALC-DRAWING-PANE   (bitmap-pane) ())

(defmethod DEFAULT-PANE-CLASS ((stream gcalc-drawing-window))
  'gcalc-drawing-pane)

;;; Making it pop-able.
(defmethod POP-UP-DIALOG ((win gcalc-drawing-window) &optional stream)
  (let ((pane (frame-child win)))
    (unwind-protect
        (progn
          (unless (and (eq *gcalc-draw-icon* (last-icon win))
                       (bitmap-ok-p *gcalc-draw-icon*))
            (if (bitmap-ok-p *gcalc-draw-icon*)
              (copy-bitmap-to-window pane (saved-bitmap *gcalc-draw-icon*))
              (progn
                (say "Drawing...")
                (with-delayed-redraw (win)
                  (with-background-color (win light-gray)
                    (clear-page win)
                    (draw-exp pane (evaluate *gcalc-draw-icon*)
                              0.0 0.0 0.0 1.0 1.0 1.0 0.0)))
                (unsay)
                (setf (saved-bitmap *gcalc-draw-icon*)
                      (create-bitmap-copy
                       pane (saved-bitmap *gcalc-draw-icon*)))
                (setf (bitmap-ok-p *gcalc-draw-icon*) t)))
            (setf (last-icon win) *gcalc-draw-icon*))
          (setf (window-state win) :normal)
          (select-window win)
          (setf (gcalc-window-done win) nil)
          (catch 'gcalc-draw-done
            (process-events)))
      (setf (window-state win) :hidden)))
  t)

;;; Exit the dialog box on a click.

(defmethod EVENT ((win gcalc-drawing-window) event buttons data time)
  (select event
    (mouse-out
     (unless (gcalc-window-done win)
       (throw 'gcalc-draw-done t)))
    (t (call-next-method))))

(defmethod EVENT ((win gcalc-drawing-pane) event buttons data time)
  (select event
    (mouse-left-down
     (setf (gcalc-window-done (window-parent win)) t)
     (throw 'gcalc-draw-done t))
    (mouse-right-down
     (setf (gcalc-window-done (window-parent win)) t)
     (throw 'gcalc-draw-done t))
    (t (call-next-method))))

;;; Create the pane.
(defun MAKE-GCALC-DRAW-PALETTE ()
  (setf *gcalc-draw-window*
        (open-stream 'gcalc-drawing-window *booms-app* :io
                     :title           "GCalc drawings"
                     :window-border   :palette
                     :window-state    :shrunk
                     :user-closable   nil
                     :user-moveable   t
                     :user-resizable  nil
                     :user-scrollable nil
                     :user-shrinkable nil
                     :overlapped      t
                     :pop-up-p        nil
                     :window-interior (make-box
                                       0 0
                                       +3d-full-size-x+ +3d-full-size-y+))))


;;;-------------------------------------------------------------------------;;;
;;;  B I T M A P  utility functions                                         ;;;
;;;-------------------------------------------------------------------------;;;

;;; This function is used to create a new Windows bitmap object that holds a
;;; copy of the given window's bitmap.  "window" should be a bitmap-pane.
(defun CREATE-BITMAP-COPY (window &optional bitmap)
  (let* ((sizex  (window-interior-width window))
         (sizey  (window-interior-height window))
         (source (device-bitmap window))
         (result (or bitmap (win:CreateCompatibleBitmap source sizex sizey)))
         (dest   (win:CreateCompatibleDC source)))
    (win:SelectObject dest result)
    (win:BitBlt dest 0 0 sizex sizey source 0 0 win:SRCCOPY)
    (win:DeleteDC dest)
    result))

;;; This function is used to draw Windows bitmap object on a given window.
;;; "window" should be a bitmap-pane.
(defun COPY-BITMAP-TO-WINDOW (window bitmap)
  (let* ((sizex  (window-interior-width window))
         (sizey  (window-interior-height window))
         (dest   (device-bitmap window))
         (source (win:CreateCompatibleDC dest)))
    (win:SelectObject source bitmap)
    (win:BitBlt dest 0 0 sizex sizey source 0 0 win:SRCCOPY)
    (win:DeleteDC source)
    t))


;;;-------------------------------------------------------------------------;;;
;;;  G C A L C  package registration                                        ;;;
;;;-------------------------------------------------------------------------;;;

(defun INITIALIZE-GCALC-PACKAGE ()
  (register-type :gcalc)
  (make-basic-color-tab)
  (make-gcalc-draw-palette)
  (add-node-class 'left-right        :constructors)
  (add-node-class 'top-bottom        :constructors)
  (add-node-class 'front-back        :constructors)
  (add-node-class 'right-left        :constructors)
  (add-node-class 'bottom-top        :constructors)
  (add-node-class 'back-front        :constructors)
  (add-node-class 'lambda-func       :operators)
  (add-node-class 'apply-func        :operators)
  (add-node-class 'basic-color       :colors)
  (add-node-class 'black-const       :colors)
  (add-node-class 'gray-const        :colors)
  (add-node-class 'lt-gray-const     :colors)
  (add-node-class 'white-const       :colors)
  (add-node-class 'red-const         :colors)
  (add-node-class 'green-const       :colors)
  (add-node-class 'blue-const        :colors)
  (add-node-class 'yellow-const      :colors)
  (add-node-class 'cyan-const        :colors)
  (add-node-class 'magenta-const     :colors)
  (add-node-class 'dk-red-const      :colors)
  (add-node-class 'dk-green-const    :colors)
  (add-node-class 'dk-blue-const     :colors)
  (add-node-class 'dk-yellow-const   :colors)
  (add-node-class 'dk-cyan-const     :colors)
  (add-node-class 'dk-magenta-const  :colors)
  (add-node-class 'transparent-const :colors))

(defmethod FRAME-TYPE ((type (eql :gcalc)))
  'gcalc-frame)

(defmethod PANE-TYPE ((type (eql :gcalc)))
  'gcalc-doc)


;;;-------------------------------------------------------------------------;;;
;;;  D O N E !                                                              ;;;
;;;-------------------------------------------------------------------------;;;

(provide :GCALC)

