;;;                    -*- Mode: Lisp; Package: MISC -*-                    ;;;
;;; ----------------------------------------------------------------------- ;;;
;;; File:         misc.lsp                                                  ;;;
;;; Description:  BOOMS miscellaneous functions                             ;;;
;;; Author:       Eli Barzilay - eli@cs.bgu.ac.il                           ;;;
;;; Date:         24-9-96                                                   ;;;
;;; Package:      MISC                                                      ;;;
;;; ----------------------------------------------------------------------- ;;;

#|

This file is part of the BOOMS system:
+-----------------------------------------------------------------------------+
|                                                                             |
| /######   /#####   /#####  /##     /##  /#####                              |
| /##///## /##///## /##///## /###   /### /##///##                        /--- |
| /##  /## /##  /## /##  /## /#### /#### /##  ///          /------/     /     |
| /######/ /##  /## /##  /## /##/####/## //#####          /      /     /      |
| /##///## /##  /## /##  /## /##//##//##  /////##        /------/ __  /       |
| /##  /## /##  /## /##  /## /## /// /##      /##   __  /  __  / /  \/        |
| /##  /## /##  /## /##  /## /##     /## /##  /##  /  \/  /  \/  \__/         |
| /######/ //#####/ //#####/ /##     /## //#####/  \__/   \__/                |
| ///////   //////   //////  ///     ///  //////                              |
|                                                                             |
|                   Booms Object Oriented Music Structures.                   |
|Eli Barzilay:                                                   Maze is Life!|
+-----------------------------------------------------------------------------+

|#


;;;-------------------------------------------------------------------------;;;
;;;  M I S C  file                                                          ;;;
;;;-------------------------------------------------------------------------;;;

(in-package :misc)


;;;-------------------------------------------------------------------------;;;
;;;  U T I L I T I E S   &   M I S C   definitions                          ;;;
;;;-------------------------------------------------------------------------;;;

;;; Global constant used to indicate no value.
(defconstant +NO-VALUE+ :<no-value>)

;;; Iteration over a tree, the list case is not done using map to allow mapping
;;; over dotted lists.  Can be used only on a structure of lists and vectors.
(defun MAPTREE (func tree)
  (cond
    ((vectorp tree) (map 'vector #'(lambda (x) (maptree func x)) tree))
    ((listp   tree) (if tree
                      (cons (maptree func (first tree))
                            (maptree func (rest  tree)))
                      nil))
    (t (funcall func tree))))

;;; Making a copy of simple values.
(defmethod MAKE-COPY ((x t))
  x)
;;; Making a copy - the general case for [nested] sequences.
(defmethod MAKE-COPY ((s sequence))
  (maptree #'make-copy s))
;;; Making a copy of a string.
(defmethod MAKE-COPY ((str string))
  (copy-seq str))

;;; Appending two symbols to a symbol.
(defun SYMBOL-APPEND (s1 s2 &optional (package *package*))
  (intern (concatenate 'string (string s1) (string s2)) package))

;;; This is used by the defnode macros, it gets a superclasses (names) list and
;;; another class name, and its output is insured to inherit from that class
;;; (if no subclasses of it, it is appended).
(defun INSURE-SUPERCLASS (superclasses class)
  (if (not (some #'(lambda (s) (subtypep s class)) superclasses))
    (append superclasses (list class))
    superclasses))

;;; Convert a pathname to a string.
(defun PATHNAME-TO-STRING (pathname)
  (concatenate 'string (pathname-name pathname) "."
                       (pathname-type pathname)))

;;; Push an item to the tail of a list destructively.
(defmacro PUSHLAST (item place)
  `(setf ,place (nconc ,place (list ,item))))

;;; Evaluate forms in reverse order.
(defmacro PROGREV (&body forms)
  `(progn ,@(reverse forms)))


;;;-------------------------------------------------------------------------;;;
;;;  D O N E !                                                              ;;;
;;;-------------------------------------------------------------------------;;;

(provide :MISC)

