;;;                   -*- Mode: Lisp; Package: MUSIC -*-                    ;;;
;;; ----------------------------------------------------------------------- ;;;
;;; File:         music.lsp                                                 ;;;
;;; Description:  BOOMS musical documents                                   ;;;
;;; Author:       Eli Barzilay - eli@cs.bgu.ac.il                           ;;;
;;; Date:         29-12-96                                                  ;;;
;;; Package:      MUSIC                                                     ;;;
;;; ----------------------------------------------------------------------- ;;;

#|

This file is part of the BOOMS system:
+-----------------------------------------------------------------------------+
|                                                                             |
| /######   /#####   /#####  /##     /##  /#####                              |
| /##///## /##///## /##///## /###   /### /##///##                        /--- |
| /##  /## /##  /## /##  /## /#### /#### /##  ///          /------/     /     |
| /######/ /##  /## /##  /## /##/####/## //#####          /      /     /      |
| /##///## /##  /## /##  /## /##//##//##  /////##        /------/ __  /       |
| /##  /## /##  /## /##  /## /## /// /##      /##   __  /  __  / /  \/        |
| /##  /## /##  /## /##  /## /##     /## /##  /##  /  \/  /  \/  \__/         |
| /######/ //#####/ //#####/ /##     /## //#####/  \__/   \__/                |
| ///////   //////   //////  ///     ///  //////                              |
|                                                                             |
|                   Booms Object Oriented Music Structures.                   |
|Eli Barzilay:                                                   Maze is Life!|
+-----------------------------------------------------------------------------+

|#


;;;-------------------------------------------------------------------------;;;
;;;  M U S I C  file                                                        ;;;
;;;-------------------------------------------------------------------------;;;

(in-package :music)


;;;-------------------------------------------------------------------------;;;
;;;  M U S I C  window classes declaration                                  ;;;
;;;-------------------------------------------------------------------------;;;

(defclass MUSIC-FRAME (doc-frame) ())

(defclass MUSIC-DOC (document) ())

(defmethod DOCUMENT-ICON (type) (booms-icon 2))


;;;-------------------------------------------------------------------------;;;
;;;  M U S I C  icons                                                       ;;;
;;;-------------------------------------------------------------------------;;;

(defclass MUSIC-ICON () ())

(defmethod ICON-EVAL ((icon music-icon))
  (let ((result (call-next-method)))
    (play (evaluate icon))
    result))

;;; The icon label height.
(defconstant +MUSIC-ICON-LABEL-HEIGHT+ 10)

;;; Drawing some icons includes a string, use this to draw it.
(defmethod ICON-DRAW-MUSIC-STRING ((icon icon) (str string))
  (let ((pos   #.(make-position 0 0))
        (box   #.(make-box 0 0 0 0))
        (dkred #.(make-rgb :red 128))
        (doc   (icon-doc icon)))
    (nmake-position pos
                    +icon-margin+
                    #.(+ +icon-margin+
                         (- +win-icon-size+ +music-icon-label-height+)))
    (nmake-box-relative-from-corner
     box (nposition+ pos (icon-pos icon))
     +win-icon-size+ #.(+ 2 +music-icon-label-height+))
    (with-font (doc #.(make-font :modern :arial 14 '(:bold)))
      (with-foreground-color (doc dkred)
        (fill-box doc box))
      (with-foreground-color (doc yellow)
        (with-background-color (doc dkred)
          (draw-string-in-box
           doc str 0 (length str) box :center :center nil))))))


;;;-------------------------------------------------------------------------;;;
;;;  N O T A T E D  values                                                  ;;;
;;;-------------------------------------------------------------------------;;;

;;; Notated values follows Mira's note algebra.
(defclass NOTATED (savable)
  ((name   :accessor name   :initarg :name)
   (shift  :accessor shift  :initarg :shift)
   (octave :accessor octave :initarg :octave)))

(defmethod LOAD-SLOTS append ((x notated))
  '(name shift octave))

;;; Making a copy of a notated object.
(defmethod MAKE-COPY ((n notated))
  (make-instance (type-of n)
                 :name   (name   n)
                 :shift  (shift  n)
                 :octave (octave n)))

;;; Calculate the numeric value for a notated triplet.
(defun CALCULATE-NUMERIC-VALUE (name shift octave)
  (+ (* 2 name) (if (> name 2) -1 0)
     shift
     (* 12 octave)))

;;; Get the numeric equivalent value for a notated object.
(defmethod NVALUE ((n notated))
  (calculate-numeric-value (name n) (shift n) (octave n)))

;;; Adding two <name,shift,octave> notated triplets, giving a notated value.
(defmethod NADD ((n1 notated) (n2 notated))
  (multiple-value-bind (div rem)
      (floor (+ (name n1) (name n2) (* 7 (octave n1)) (* 7 (octave n2))) 7)
    (make-instance 'notated
                   :name   rem
                   :shift  (- (+ (nvalue n1) (nvalue n2))
                              (calculate-numeric-value rem 0 div))
                   :octave div)))

;;; Subtracting - similar to the above.
(defmethod NSUB ((n1 notated) (n2 notated))
  (multiple-value-bind (div rem)
      (floor (+ (- (name n1) (name n2))
                (- (* 7 (octave n1)) (* 7 (octave n2))))
             7)
    (make-instance 'notated
                   :name   rem
                   :shift  (- (- (nvalue n1) (nvalue n2))
                              (calculate-numeric-value rem 0 div))
                   :octave div)))

;;; Notated notes.

(defclass NOTE (notated) ())

(defmethod MAKE-NOTE ((name integer) (shift integer) (octave integer))
  (make-instance 'note :name name :shift shift :octave octave))

;;; Notated intervals.

(defclass INTERVAL (notated) ())

(defmethod MAKE-INTERVAL ((name integer) (shift integer) (octave integer))
  (make-instance 'interval :name name :shift shift :octave octave))

;;; Addition of notated values.
(defmethod B+ ((x note    ) (y interval)) (change-class (nadd x y) 'note))
(defmethod B+ ((x interval) (y note    )) (change-class (nadd x y) 'note))
(defmethod B+ ((x interval) (y interval)) (change-class (nadd x y) 'interval))

;;; Subtraction of notated values.
(defmethod B- ((x note    ) (y interval)) (change-class (nsub x y) 'note))
(defmethod B- ((x note    ) (y note    )) (change-class (nsub x y) 'interval))
(defmethod B- ((x interval) (y interval)) (change-class (nsub x y) 'interval))


;;;-------------------------------------------------------------------------;;;
;;;  R E A D E R - M A C R O  definition                                    ;;;
;;;-------------------------------------------------------------------------;;;
;;; This gets a string (from a reader macro) representing a note and generates
;;; a notated value - note or interval.
;;; Syntax:
;;;   "#_" [rel-octave] name [alteration] [abs-octave]
;;; where:
;;;   rel-octave - any number of "+"'s and "-"'s, default octave is 5 for note
;;;                and 0 for an interval
;;;   name       - one symbol of  do re mi fa sol (or so) la si (or ci)
;;;                or of prime (or first) second third fourth fifth sixth
;;;                   seventh (these will create an interval)
;;;   abs-octave - positive integer (1-12), setting this will ignore rel-octave

(defconstant NOTES-PACKAGE (or (find-package :NOTES) (make-package :NOTES)))

(set-dispatch-macro-character #\# #\_
  #'(lambda (stream char arg)
      (declare (ignore char arg))
      (let ((sym (let ((*package* notes-package)) (read stream t))))
        (unless (boundp sym)
          (eval
           `(defconstant ,sym ,(string-to-notated (princ-to-string sym)))))
        sym)))

(defun STRING-TO-NOTATED (str)
  (let* ((strlen     (length str))
         (type       nil)
         (name       nil)
         (shift      0)
         (octave     0)
         (i0         0)
         (i1         0))
    (macrolet ((nextstr (pred)
                 `(progn
                    (shiftf i0 i1
                            (position-if-not ,pred str :start i1))
                    (or i1 (setf i1 strlen)))))
      ;; next "+" / "-" substring
      (nextstr #'(lambda (ch) (member ch '(#\+ #\-))))
      ;; inc/dec octave by that substring
      (do ((i i0 (1+ i)))
          ((= i i1))
        (case (char str i)
          ((#\+) (incf octave))
          ((#\-) (decf octave))))
      ;; next is the note name
      (nextstr #'alpha-char-p)
      (if (eq i0 i1)
        ;; should have some name
        (error "You must specify a note name in ~s" str)
        (let ((name-symbol (intern (string-upcase (subseq str i0 i1)))))
          ;; set object type
          (setf type
                (case name-symbol
                  ((do re mi fa sol so la si ci)
                   'n)
                  ((prime first second third fourth fifth sixth seventh)
                   'i)))
          ;; translate note name to note value
          (setf name
                (case name-symbol
                  ((do     prime first) 0)
                  ((re     second     ) 1)
                  ((mi     third      ) 2)
                  ((fa     fourth     ) 3)
                  ((sol so fifth      ) 4)
                  ((la     sixth      ) 5)
                  ((si ci  seventh    ) 6)))))
      (if (eq type 'n) (incf octave 5))
      ;; next "#" / "/" substring
      (nextstr #'(lambda (ch) (member ch '(#\# #\/))))
      ;; inc/dec shift by that substring
      (do ((i i0 (1+ i)))
          ((= i i1))
        (case (char str i)
          ((#\#) (incf shift))
          ((#\/) (decf shift))))
      ;; next is an absolute octave number
      (nextstr #'digit-char-p)
      ;; if specified, set this number
      (when (> i1 i0)
        (setf octave (parse-integer str :start i0 :end i1)))
      (unless (= i1 strlen)
        (error "Wrong note: ~s" str))
      (case type
        ((n) (make-note     name shift octave))
        ((i) (make-interval name shift octave))))))

;;; These will print notated values so they can be read back.

(defmethod PRINT-OBJECT ((n note) stream)
  (format stream
          "#_~[Do~;Re~;Mi~;Fa~;Sol~;La~;Si~]~,,v,v@a"
          (name n) (abs (shift n)) (if (> (shift n) 0) #\# #\/) (octave n)))

(defmethod PRINT-OBJECT ((n interval) stream)
  (format stream
          "#_~,,v,v@a~:[~:(~:r~)~;Prime~*~]"
          (abs (octave n)) (if (> (octave n) 0) #\+ #\-) ""
          (zerop (name n)) (1+ (name n))
          (abs (shift n)) (if (> (shift n) 0) #\# #\/) ""))


;;;-------------------------------------------------------------------------;;;
;;;  S O U N D  values                                                      ;;;
;;;-------------------------------------------------------------------------;;;

;;; Sound values contain note pitch, velocity, duration and a start time.  The
;;; start time is used only in FMP's.
(defclass SOUND (savable)
  ((pitch    :accessor pitch    :initarg :pitch)
   (velocity :accessor velocity :initarg :velocity)
   (duration :accessor duration :initarg :duration)
   (start    :accessor start    :initarg :start :initform 0)))

(defmethod LOAD-SLOTS append ((x sound))
  '(pitch velocity duration start))

;;; Note constructor.
(defmethod MAKE-SOUND ((pitch note) (velocity real) (duration rational))
  (make-instance 'sound :pitch pitch :velocity velocity :duration duration))

;;; A representation to recreate an equivalent sound.
(defmethod PRINT-OBJECT ((sound sound) stream)
  (format stream "(make-sound ~s ~s ~s)"
          (pitch sound)
          (velocity sound)
          (duration sound)))

;;; Copying a note.
(defmethod MAKE-COPY ((sound sound))
  (make-instance 'sound
                 :pitch    (make-copy (pitch sound))
                 :velocity (velocity sound)
                 :duration (duration sound)
                 :start    (start    sound)))


;;;-------------------------------------------------------------------------;;;
;;;  F M P  values                                                          ;;;
;;;-------------------------------------------------------------------------;;;

;;; An FMP is a flat music piece - it holds a list of sound objects, with the
;;; start times set (relative to 0).  The 'end' slot is for calculation
;;; optimization.
(defclass FMP (savable)
  ((sounds :accessor sounds  :initarg :sounds)
   (end    :accessor fmp-end :initarg :fmp-end)))

(defmethod LOAD-SLOTS append ((x fmp))
  '(sounds end))

;;; This is not conventional as it cannot be read back in.
(defmethod PRINT-OBJECT ((fmp fmp) stream)
  (princ "{" stream)
  (do ((sounds (sounds fmp) (rest sounds))
       (i 10))
      ((or (null sounds) (zerop i)))
    (let ((sound (first sounds)))
      (prin1 (pitch sound) stream)
      (princ ":" stream)
      (prin1 (duration sound) stream)
      (princ "@" stream)
      (prin1 (start sound) stream))
    (when (rest sounds)
      (princ ", " stream)
      (when (zerop (decf i))
        (princ "..." stream))))
  (princ "}" stream))

;;; Create a copy of an FMP.
(defmethod MAKE-COPY ((fmp fmp))
  (make-instance 'fmp
                 :sounds  (make-copy (sounds fmp))
                 :fmp-end (fmp-end fmp)))

;;; Make an FMP out of a sound.
(defmethod MAKE-FMP ((sound sound))
  (make-instance 'fmp
                 :sounds (list sound)
                 :fmp-end (+ (start sound) (duration sound))))

;;; Getting the FMP start time.
(defmethod FMP-START ((fmp fmp))
  (let ((sounds (sounds fmp)))
    (if (null sounds)
      0
      (start (first sounds)))))

;;; Setting the FMP start time - move the entire FMP.
(defmethod (setf FMP-START) ((newval real) (fmp fmp))
  (let ((delta (- newval (fmp-start fmp))))
    (dolist (s (sounds fmp))
      (incf (start s) delta))
    (incf (fmp-end fmp) delta))
  newval)

;;; The duration of an FMP.
(defmethod FMP-DURATION ((fmp fmp))
  (- (fmp-end fmp) (fmp-start fmp)))

;;; Merge the second FMP into the first - sequentially.
(defmethod SEQUENTIAL-MERGE ((fmp fmp) (fmp1 fmp))
  (let ((fmp0 (make-copy fmp1)))
    (setf (fmp-start fmp0) (fmp-end fmp))
    (setf (fmp-end fmp) (fmp-end fmp0))
    (setf (sounds fmp) (nconc (sounds fmp) (sounds fmp0)))
    fmp))

;;; Merge the second FMP into the first - simultaneously.
(defmethod SIMULTANEOUS-MERGE ((fmp fmp) (fmp1 fmp))
  (let ((slist (make-copy (sounds fmp1))))
    (setf (fmp-end fmp) (max (fmp-end fmp) (fmp-end fmp1)))
    (setf (sounds fmp)
          (sort (nconc (sounds fmp) slist) #'< :key #'start))
    fmp))

;;; Merge the second FMP into the first - merge fmp1 at the given time1.
(defmethod GENERAL-MERGE ((fmp fmp) (time1 real) (fmp1 fmp))
  (let ((fmp0 (make-copy fmp1)))
    (setf (fmp-start fmp0) time1)
    (setf (fmp-end fmp) (max (fmp-end fmp) (fmp-end fmp0)))
    (setf (sounds fmp)
          (sort (nconc (sounds fmp) (sounds fmp0)) #'< :key #'start))
    fmp))

;;; Expand the FMP playing time.
(defmethod EXPAND-FMP ((fmp fmp) (factor real))
  (setf (fmp-end fmp) (* factor (fmp-end fmp)))
  (dolist (sound (sounds fmp))
    (setf (start sound) (* factor (start sound)))
    (setf (duration sound) (* factor (duration sound))))
  fmp)

;;; Shift an FMP by a given interval.
(defmethod SHIFT-FMP ((fmp fmp) (interval interval))
  (dolist (sound (sounds fmp))
    (setf (pitch sound) (B+ interval (pitch sound))))
  fmp)

;;; Flip an FMP up-side-down.
(defmethod FLIP-FMP ((fmp fmp))
  (when (sounds fmp)
    (let* ((min    (pitch (first (sounds fmp))))
           (minval (nvalue min))
           (max    min)
           (maxval minval))
      (dolist (sound (rest (sounds fmp)))
        (let ((sval (nvalue (pitch sound))))
          (when (< sval minval) (setf min (pitch sound)))
          (when (> sval maxval) (setf max (pitch sound)))))
      (dolist (sound (sounds fmp))
        (setf (pitch sound) (B+ (B- max (pitch sound)) min)))))
  fmp)

;;; Reverse an FMP in time.
(defmethod REVERSE-FMP ((fmp fmp))
  (let ((fmp-duration (fmp-duration fmp)))
    (dolist (sound (sounds fmp))
      (setf (start sound) (- fmp-duration (start sound) (duration sound))))
    (setf (sounds fmp) (sort (sounds fmp) #'< :key #'start)))
  fmp)

;;; Normalize the FMP velocity so the maximum is 1.0.
(defmethod FMP-NORMALIZE-VELOCITY ((fmp fmp))
  (when (sounds fmp)
    (let ((maxvel (velocity (first (sounds fmp)))))
      (dolist (sound (rest (sounds fmp)))
        (when (> (velocity sound) maxvel) (setf maxvel (velocity sound))))
      (when (> maxvel 1.0)
        (dolist (sound (sounds fmp))
          (setf (velocity sound) (/ (velocity sound) maxvel))))))
  fmp)

;;; Change an FMP volume by a factor.  The factor can be greater than 1 -
;;; fmp-normalize-velocity can be used to fix this (for example, before
;;; playing).
(defmethod FMP-CHANGE-VELOCITY ((fmp fmp) (vel real))
  (dolist (sound (sounds fmp))
    (setf (velocity sound) (* vel (velocity sound))))
  fmp)

;;; Change the volume smoothly - given start/end factors.
(defmethod FMP-SMOOTH-VELOCITY ((fmp fmp) (vel1 real) (vel2 real))
  (let ((fmp-duration (fmp-duration fmp)))
    (dolist (sound (sounds fmp))
      (setf (velocity sound)
            (* (+ vel1
                  (* (- vel2 vel1)
                     (/ (+ (start sound) (/ (duration sound) 2))
                        fmp-duration)))
               (velocity sound)))))
  fmp)

;;; Generalized versions of the above seq/sim methods.

(defmethod SEQUENTIAL ((fmp-list list))
  (cond ((null fmp-list)        (make-instance 'fmp :sounds nil :fmp-end 0))
        ((null (rest fmp-list)) (make-copy (first fmp-list)))
        (t                      (reduce #'sequential-merge fmp-list))))

(defmethod SIMULTANEOUS ((fmp-list list))
  (cond ((null fmp-list)        (make-instance 'fmp :sounds nil :fmp-end 0))
        ((null (rest fmp-list)) (make-copy (first fmp-list)))
        (t                      (reduce #'simultaneous-merge fmp-list))))


;;;-------------------------------------------------------------------------;;;
;;;  M U S I C  operators                                                   ;;;
;;;-------------------------------------------------------------------------;;;

;;; FMP operators.

(deficon SEQUENTIAL (mps list fmp) (music-icon) ()
  (evaluate fmp := (sequential mps)))
(defmethod ICON-ICON ((icon sequential)) (booms-icon 9))

(deficon SIMULTANEOUS (mps list fmp) (music-icon) ()
  (evaluate fmp := (simultaneous mps)))
(defmethod ICON-ICON ((icon simultaneous)) (booms-icon 10))

(deficon GENERAL-CONCAT ((mp fmp) (time1 real) (mp1 fmp)) (music-icon) ()
  (evaluate fmp := (general-merge mp time1 mp1)))
(defmethod ICON-ICON ((icon general-concat)) (booms-icon 12))

(deficon TIME-EXPAND ((mp fmp) (factor real)) (music-icon) ()
  (evaluate fmp := (expand-fmp mp factor)))
(defmethod ICON-ICON ((icon time-expand)) (booms-icon 13))

(deficon TRANSPOSE ((mp fmp) (interval interval)) (music-icon) ()
  (evaluate fmp := (shift-fmp mp interval)))
(defmethod ICON-ICON ((icon transpose)) (booms-icon 11))

(deficon FLIP ((mp fmp)) (music-icon) ()
  (evaluate fmp := (flip-fmp mp)))
(defmethod ICON-ICON ((icon flip)) (booms-icon 19))

(deficon REVERSE ((mp fmp)) (music-icon) ()
  (evaluate fmp := (reverse-fmp mp)))
(defmethod ICON-ICON ((icon reverse)) (booms-icon 18))

(deficon VELOCITY ((mp fmp) (velocity real)) (music-icon) ()
  (evaluate fmp := (fmp-change-velocity mp velocity)))
(defmethod ICON-ICON ((icon velocity)) (booms-icon 16))

(deficon CRESCHENDO ((mp fmp) (velocity1 real) (velocity2 real))
  (music-icon) ()
  (evaluate fmp := (fmp-smooth-velocity mp velocity1 velocity2)))
(defmethod ICON-ICON ((icon creschendo)) (booms-icon 17))


;;;-------------------------------------------------------------------------;;;
;;;  M U S I C  icons                                                       ;;;
;;;-------------------------------------------------------------------------;;;

;;; An interval leaf.

(defleaf ICONIC-INTERVAL ()
  ((type-out :allocation :class :initform 'interval)))

(set-booms-name iconic-interval 'interval)

(defmethod ICON-ICON ((icon iconic-interval)) (booms-icon 20))

;;; Make it behave like a notated object.

(defmethod NAME   ((icon iconic-interval)) (name   (arguments icon)))

(defmethod SHIFT  ((icon iconic-interval)) (shift  (arguments icon)))

(defmethod OCTAVE ((icon iconic-interval)) (octave (arguments icon)))

(defmethod (setf NAME) ((newval integer) (icon iconic-interval))
  (setf (name (arguments icon)) newval)
  (forget-value icon))

(defmethod (setf SHIFT) ((newval integer) (icon iconic-interval))
  (setf (shift (arguments icon)) newval)
  (forget-value icon))

(defmethod (setf OCTAVE) ((newval integer) (icon iconic-interval))
  (setf (octave (arguments icon)) newval)
  (forget-value icon))

;;; Draw the interval name.
(defmethod ICON-DRAW ((icon iconic-interval))
  (call-next-method)
  (icon-draw-music-string icon (subseq (prin1-to-string (arguments icon)) 2)))

;;; Ask user for initial value using the notated pane.
(defmethod LEAF-INIT-VAL/PANE ((icon iconic-interval))
  (values #_prime :notated))

;;; A general sound class - evaluates to an FMP.

(defleaf ICONIC-SOUND (music-icon)
  ((type-out :allocation :class :initform 'fmp)))

(set-booms-name iconic-sound 'note)

(defmethod ICON-ICON ((icon iconic-sound)) (booms-icon 8))

;;; Make it behave like a notated / sound object.

(defmethod NAME ((icon iconic-sound))
  (name (pitch (first (sounds (arguments icon))))))

(defmethod SHIFT ((icon iconic-sound))
  (shift (pitch (first (sounds (arguments icon))))))

(defmethod OCTAVE ((icon iconic-sound))
  (octave (pitch (first (sounds (arguments icon))))))

(defmethod PITCH ((icon iconic-sound))
  (pitch (first (sounds (arguments icon)))))

(defmethod DURATION ((icon iconic-sound))
  (fmp-duration (arguments icon)))

(defmethod VELOCITY ((icon iconic-sound))
  (velocity (first (sounds (arguments icon)))))

(defmethod (setf NAME) ((newval integer) (icon iconic-sound))
  (setf (name (pitch (first (sounds (arguments icon))))) newval)
  (forget-value icon))

(defmethod (setf SHIFT) ((newval integer) (icon iconic-sound))
  (setf (shift (pitch (first (sounds (arguments icon))))) newval)
  (forget-value icon))

(defmethod (setf OCTAVE) ((newval integer) (icon iconic-sound))
  (setf (octave (pitch (first (sounds (arguments icon))))) newval)
  (forget-value icon))

(defmethod (setf PITCH) ((newval real) (icon iconic-sound))
  (setf (pitch (first (sounds (arguments icon)))) newval)
  (forget-value icon))

(defmethod (setf DURATION) ((newval real) (icon iconic-sound))
  (setf (duration (first (sounds (arguments icon)))) newval)
  (setf (fmp-end (arguments icon)) newval)
  (forget-value icon))

(defmethod (setf VELOCITY) ((newval real) (icon iconic-sound))
  (setf (velocity (first (sounds (arguments icon)))) newval)
  (forget-value icon))

;;; Describing an iconic-leaf with a string.
(defmethod ICON-DESCRIBE ((icon iconic-sound) &optional socket)
  (format nil "~:(~a~)    Pitch=~a    Duration=~a    Velocity=~a"
          (icon-describe-name icon)
          (pitch    icon)
          (duration icon)
          (velocity icon)))

;;; Draw the sound pitch name.
(defmethod ICON-DRAW ((icon iconic-sound))
  (call-next-method)
  (icon-draw-music-string
   icon (subseq (format nil "~a~:[~*~;:~a~]"
                        (pitch icon)
                        (integerp (/ (duration icon)))
                        (/ (duration icon)))
                2)))

;;; Ask user for initial value using the notated pane.
(defmethod LEAF-INIT-VAL/PANE ((icon iconic-sound))
  (values (make-fmp (make-sound #_do 1.0 1/4)) :notated))

;;; A rest object.

(defleaf ICONIC-REST (iconic-sound) ())

(set-booms-name iconic-rest 'rest)

(defmethod ICON-ICON ((icon iconic-rest)) (booms-icon 7))

;;; Ask user for initial value using the sound pane.
(defmethod LEAF-INIT-VAL/PANE ((icon iconic-rest))
  (values (make-fmp (make-sound (make-note 0 0 0) 0 1/4)) :sound))

(defmethod ICON-DRAW ((icon iconic-rest))
  (call-next-method)
  (icon-draw-music-string icon (prin1-to-string (duration icon))))

;;; Getting the duration of an FMP.

(deficon FMP-DURATION ((mp fmp)) () ()
  (evaluate real := (fmp-duration mp)))

(set-booms-name fmp-duration 'duration)

(defmethod ICON-DRAW ((icon fmp-duration))
  (call-next-method)
  (icon-draw-music-string icon "DUR"))

;;; Adding / multiplying numbers.

(deficon ICONIC-ADD (nums list real) () ()
  (evaluate real := (apply #'+ nums)))

(set-booms-name iconic-add 'add)

(defmethod ICON-DRAW ((icon iconic-add))
  (call-next-method)
  (icon-draw-music-string icon "+"))

(deficon ICONIC-MULTIPLY (nums list real) () ()
  (evaluate real := (apply #'* nums)))

(set-booms-name iconic-multiply 'mult)

(defmethod ICON-DRAW ((icon iconic-multiply))
  (call-next-method)
  (icon-draw-music-string icon "*"))

;;; General number constants.

(defleaf ICONIC-NUMBER ()
  ((type-out :allocation :class :initform 'real)))

(set-booms-name iconic-number 'number)

(defmethod ICON-ICON ((icon iconic-number)) (booms-icon 24))

;;; Ask user for initial value using the leaf pane.
(defmethod LEAF-INIT-VAL/PANE ((icon iconic-number))
  (values 0 :leaf))

;;; Draw the number.
(defmethod ICON-DRAW ((icon iconic-number))
  (call-next-method)
  (icon-draw-music-string icon (prin1-to-string (arguments icon))))

;;; General factor constants - numbers from 0 to 1.

(defleaf ICONIC-FACTOR (iconic-number)
  ((type-out :allocation :class :initform '(real 0 1))))

(set-booms-name iconic-factor 'factor)

;;; Ask user for initial value using the factor pane.
(defmethod LEAF-INIT-VAL/PANE ((icon iconic-factor))
  (values 1 :factor))

(defmethod ICON-DRAW ((icon iconic-factor))
  (call-next-method)
  (let ((pos   #.(make-position 0 0))
        (box   #.(make-box 0 0 0 0))
        (box1  #.(make-box 0 0 0 0))
        (dkred #.(make-rgb :red 128))
        (doc   (icon-doc icon)))
    (nmake-position pos
                    +icon-margin+
                    #.(+ +icon-margin+
                         (- +win-icon-size+ +music-icon-label-height+)))
    (nmake-box-relative-from-corner
     box (nposition+ pos (icon-pos icon))
     +win-icon-size+ #.(+ 2 +music-icon-label-height+))
    (nmake-box-relative-from-corner
     box1 pos
     (round (* (arguments icon) +win-icon-size+))
     #.(+ 2 +music-icon-label-height+))
    (with-font (doc #.(make-font :modern :arial 14 '(:bold)))
      (with-foreground-color (doc dkred)
        (fill-box doc box))
      (with-foreground-color (doc green)
        (fill-box doc box1))
      (with-foreground-color (doc black)
        (with-line-width (doc 2)
          (draw-box doc box))))))


;;;-------------------------------------------------------------------------;;;
;;;  M U S I C  identity icon                                               ;;;
;;;-------------------------------------------------------------------------;;;

;;; The music identity icon - make it play on evaluation.
(defclass MUSIC-IDENTITY (music-icon iconic-identity) ())

(set-booms-name music-identity 'identity)

;;; Return this class for the :music doc-type.
(defmethod ICONIC-IDENTITY-CLASS-NAME ((doctype (eql :music))) 'music-identity)


;;;-------------------------------------------------------------------------;;;
;;;  M U S I C  abstractions                                                ;;;
;;;-------------------------------------------------------------------------;;;

;;; Tha music specific iconic-abstraction classes.

(defclass MUSIC-ABSTRACT
  (music-icon iconic-abstract) ())

(set-booms-name music-abstract 'abstract)

(defclass MUSIC-ABSTRACT-LEAF
  (music-icon iconic-abstract-leaf) ())

(set-booms-name music-abstract-leaf 'abstract-leaf)

(defclass MUSIC-NAMED-ABSTRACT
  (music-icon iconic-named-abstract) ())

(defclass MUSIC-NAMED-ABSTRACT-LEAF
  (music-icon iconic-named-abstract-leaf) ())

;;; This are the specialization method versions for the music abstraction
;;; classes.

(defmethod ICONIC-ABSTRACT-CLASS-NAME ((doctype (eql :music)))
  'music-abstract)

(defmethod ICONIC-ABSTRACT-LEAF-CLASS-NAME ((doctype (eql :music)))
  'music-abstract-leaf)

(defmethod ICON-NAMED-ABSTRACTION-CLASS ((doctype (eql :music))
                                         (icon music-abstract))
  'music-named-abstract)

(defmethod ICON-NAMED-ABSTRACTION-CLASS ((doctype (eql :music))
                                         (icon music-abstract-leaf))
  'music-named-abstract-leaf)

(defmethod ICON-UNNAMED-ABSTRACTION-CLASS ((doctype (eql :music))
                                           (icon music-named-abstract))
  'music-abstract)

(defmethod ICON-UNNAMED-ABSTRACTION-CLASS ((doctype (eql :music))
                                           (icon music-named-abstract-leaf))
  'music-abstract-leaf)


;;;-------------------------------------------------------------------------;;;
;;;  V A R I O U S  edit pages                                              ;;;
;;;-------------------------------------------------------------------------;;;

;;; The notated pitch/interval page.

;;; The notated widgets.
(defvar *NOTATED-NAME-COMBO-BOX*   nil)
(defvar *NOTATED-SHIFT-LISP-TEXT*  nil)
(defvar *NOTATED-OCTAVE-LISP-TEXT* nil)

(defun MAKE-NOTATED-TAB ()
  (add-icon-edit-tab :notated)
  (progrev
   (add-icon-edit-tab-widget
     :widget       'static-text
     :value        "~Name:"
     :box          (make-box 20 30 75 55)
     :tabstop      nil
     :groupstart   t)
   (setf *notated-name-combo-box*
    (add-icon-edit-tab-widget
     :widget       'combo-box
     :value        nil
     :box          (make-box 85 30 220 175)
     :range        '(0 1 2 3 4 5 6)
     :key          #'(lambda (x)
                       (nth x '("Do / Prime"  "Re / Second" "Mi / Third"
                                "Fa / Fourth" "Sol / Fifth" "La / Sixth"
                                "Si / Seventh")))
     :3d-border    t
     :tabstop      t
     :groupstart   nil
     :set-value-fn #'true))
   (add-icon-edit-tab-widget
     :widget       'static-text
     :value        "~Shift:"
     :box          (make-box 20 60 75 85)
     :tabstop      nil
     :groupstart   t)
   (setf *notated-shift-lisp-text*
    (add-icon-edit-tab-widget
     :widget       'lisp-text
     :value        nil
     :box          (make-box 85 60 220 85)
     :3d-border    t
     :tabstop      t
     :groupstart   nil
     :set-value-fn #'(lambda (text newval oldval) (integerp newval))))
   (add-icon-edit-tab-widget
     :widget       'static-text
     :value        "~Octave:"
     :box          (make-box 20 90 75 115)
     :tabstop      nil
     :groupstart   t)
   (setf *notated-octave-lisp-text*
    (add-icon-edit-tab-widget
     :widget       'lisp-text
     :value        nil
     :box          (make-box 85 90 220 115)
     :3d-border    t
     :tabstop      t
     :groupstart   nil
     :set-value-fn #'(lambda (text newval oldval)
                       (and (integerp newval)
                            (or (and (typep *icon-edit-icon* 'iconic-sound)
                                     (<= 0 newval 9))
                                (and (typep *icon-edit-icon* 'iconic-interval)
                                     (<= -5 newval 5)))))))
   (add-icon-edit-tab-widget
     :widget       'button
     :title        "~A p p l y"
     :box          (make-box 40 120 200 145)
     :3d-border    t
     :tabstop      t
     :groupstart   nil
     :set-value-fn #'(lambda (&rest r)
                       (let ((newname
                              (dialog-item-value *notated-name-combo-box*))
                             (newshift
                              (dialog-item-value *notated-shift-lisp-text*))
                             (newoctave
                              (dialog-item-value *notated-octave-lisp-text*)))
                         (setf (name   *icon-edit-icon*) newname)
                         (setf (shift  *icon-edit-icon*) newshift)
                         (setf (octave *icon-edit-icon*) newoctave)
                         (icon-reshow *icon-edit-icon*)
                         (do-redraws (icon-doc *icon-edit-icon*))
                         nil)))))

;;; Initialize the page values.

(defmethod ICON-EDIT-SETUP-PAGE ((icon iconic-sound) (id (eql :notated))
                                 &optional socket)
  (set-dialog-item-value *notated-name-combo-box*   (name   icon))
  (set-dialog-item-value *notated-shift-lisp-text*  (shift  icon))
  (set-dialog-item-value *notated-octave-lisp-text* (octave icon)))

(defmethod ICON-EDIT-SETUP-PAGE ((icon iconic-interval) (id (eql :notated))
                                 &optional socket)
  (set-dialog-item-value *notated-name-combo-box*   (name   icon))
  (set-dialog-item-value *notated-shift-lisp-text*  (shift  icon))
  (set-dialog-item-value *notated-octave-lisp-text* (octave icon)))

;;; Terminate the page.

(defmethod ICON-EDIT-DONE-PAGE ((icon iconic-sound) (id (eql :notated)) result)
  (when result
    (setf (name *icon-edit-icon*)
          (dialog-item-value *notated-name-combo-box*))
    (setf (shift *icon-edit-icon*)
          (dialog-item-value *notated-shift-lisp-text*))
    (setf (octave *icon-edit-icon*)
          (dialog-item-value *notated-octave-lisp-text*))))

(defmethod ICON-EDIT-DONE-PAGE ((icon iconic-interval) (id (eql :notated))
                                result)
  (when result
    (setf (name *icon-edit-icon*)
          (dialog-item-value *notated-name-combo-box*))
    (setf (shift *icon-edit-icon*)
          (dialog-item-value *notated-shift-lisp-text*))
    (setf (octave *icon-edit-icon*)
          (dialog-item-value *notated-octave-lisp-text*))))

;;; Return this page for iconic-interval nodes.
(defmethod ICON-TAB-IDS ((x iconic-interval))
  (list :notated :colors))

;;; Return this page for iconic-sound nodes - defined for both pages below.


;;; The sound page.

;;; The sound lisp widgets.
(defvar *SOUND-DURATION-LISP-TEXT* nil)
(defvar *SOUND-VELOCITY-LISP-TEXT* nil)

(defun MAKE-SOUND-TAB ()
  (add-icon-edit-tab :sound)
  (progrev
   (add-icon-edit-tab-widget
     :widget       'static-text
     :value        "~Duration:"
     :box          (make-box 20 35 75 60)
     :tabstop      nil
     :groupstart   t)
   (setf *sound-duration-lisp-text*
    (add-icon-edit-tab-widget
     :widget       'lisp-text
     :value        nil
     :box          (make-box 85 35 220 60)
     :3d-border    t
     :tabstop      t
     :groupstart   nil
     :set-value-fn #'(lambda (text newval oldval)
                       (and (realp newval) (plusp newval)))))
   (add-icon-edit-tab-widget
     :widget       'static-text
     :value        "~Velocity:"
     :box          (make-box 20 70 75 95)
     :tabstop      nil
     :groupstart   t)
   (setf *sound-velocity-lisp-text*
    (add-icon-edit-tab-widget
     :widget       'lisp-text
     :value        nil
     :box          (make-box 85 70 220 95)
     :3d-border    t
     :tabstop      t
     :groupstart   nil
     :set-value-fn #'(lambda (text newval oldval)
                       (and (realp newval)
                            (<= 0 newval 1)
                            (or (not (typep *icon-edit-icon* 'iconic-rest))
                                (zerop newval))))))
   (add-icon-edit-tab-widget
     :widget       'button
     :title        "~A p p l y"
     :box          (make-box 40 105 200 130)
     :3d-border    t
     :tabstop      t
     :groupstart   nil
     :set-value-fn #'(lambda (&rest r)
                       (let ((newduration
                              (dialog-item-value *sound-duration-lisp-text*))
                             (newvelocity
                              (dialog-item-value *sound-velocity-lisp-text*)))
                         (setf (duration *icon-edit-icon*) newduration)
                         (setf (velocity *icon-edit-icon*) newvelocity)
                         (do-redraws (icon-doc *icon-edit-icon*))
                         nil)))))

;;; Initialize the page values.
(defmethod ICON-EDIT-SETUP-PAGE ((icon iconic-sound) (id (eql :sound))
                                 &optional socket)
  (set-dialog-item-value *sound-duration-lisp-text* (duration icon))
  (set-dialog-item-value *sound-velocity-lisp-text* (velocity icon)))

;;; Terminate the page.
(defmethod ICON-EDIT-DONE-PAGE ((icon iconic-sound) (id (eql :sound)) result)
  (when result
    (setf (duration *icon-edit-icon*)
          (dialog-item-value *sound-duration-lisp-text*))
    (setf (velocity *icon-edit-icon*)
          (dialog-item-value *sound-velocity-lisp-text*))))

;;; Return this page for iconic-sound nodes.
(defmethod ICON-TAB-IDS ((x iconic-sound))
  (list :sound :notated :colors))

;;; An iconic rest cannot change its velocity (implemented in the dialog) or
;;; pitch.
(defmethod ICON-TAB-IDS ((x iconic-rest))
  (list :sound :colors))


;;; The factor page.

;;; The factor trackbar widget.
(defvar *FACTOR-TRACKBAR* nil)

(defun MAKE-FACTOR-TAB ()
  (add-icon-edit-tab :factor)
  (progrev
   (add-icon-edit-tab-widget
     :widget       'static-text
     :value        "~Factor"
     :box          (make-box 40 30 200 50)
     :tabstop      nil
     :groupstart   t)
   (setf *factor-trackbar*
    (add-icon-edit-tab-widget
     :widget       'trackbar
     :value        nil
     :box          (make-box 40 60 200 90)
     :range        '(0 1000)
     :tick-frequency 100
     :tick-side        :top
     :orientation      :horizontal
     :3d-border    t
     :tabstop      t
     :groupstart   nil
     :set-value-fn #'true))
   (add-icon-edit-tab-widget
     :widget       'static-text
     :value        "0.0"
     :box          (make-box 20 60 40 90)
     :tabstop      nil
     :groupstart   t)
   (add-icon-edit-tab-widget
     :widget       'static-text
     :value        "1.0"
     :box          (make-box 200 60 220 90)
     :tabstop      nil
     :groupstart   t)
   (add-icon-edit-tab-widget
     :widget       'button
     :title        "~A p p l y"
     :box          (make-box 40 100 200 130)
     :3d-border    t
     :tabstop      t
     :groupstart   nil
     :set-value-fn #'(lambda (&rest r)
                       (let ((newval (dialog-item-value *factor-trackbar*)))
                         (setf (arguments *icon-edit-icon*) (/ newval 1000.0))
                         (do-redraws (icon-doc *icon-edit-icon*))
                         nil)))))

;;; Initialize the page values.
(defmethod ICON-EDIT-SETUP-PAGE ((icon iconic-factor) (id (eql :factor))
                                 &optional socket)
  (set-dialog-item-value *factor-trackbar* (round (* (arguments icon) 1000))))

;;; Terminate the page.
(defmethod ICON-EDIT-DONE-PAGE ((icon iconic-factor) (id (eql :factor)) result)
  (when result
    (setf (arguments icon) (/ (dialog-item-value *factor-trackbar*) 1000.0))))

;;; Return this page for iconic-factor nodes.
(defmethod ICON-TAB-IDS ((x iconic-factor))
  (list :factor :colors))


;;;-------------------------------------------------------------------------;;;
;;;  P L A Y  functions                                                     ;;;
;;;-------------------------------------------------------------------------;;;

;;; Nothing to play on default objects.
(defmethod PLAY ((x t))
  t)

;;; Top-level playing of FMP.
(defmethod PLAY ((fmp fmp))
  (fmp-normalize-velocity fmp)
  (start-play)
  (dolist (sound (sounds fmp))
    (play sound))
  (end-play))

;;; Playing single sounds.
(defmethod PLAY ((sound sound))
  (play-note (nvalue (pitch sound))
             :wait     (start sound)
             :duration (duration sound)
             :velocity (velocity sound)))


;;;-------------------------------------------------------------------------;;;
;;;  M U S I C  package initialization                                      ;;;
;;;-------------------------------------------------------------------------;;;

(defun INITIALIZE-MUSIC-PACKAGE ()
  (register-type :music)
  (make-notated-tab)
  (make-sound-tab)
  (make-factor-tab)
  (add-node-class 'sequential      :music-ops)
  (add-node-class 'simultaneous    :music-ops)
  (add-node-class 'general-concat  :music-ops)
  (add-node-class 'time-expand     :music-ops)
  (add-node-class 'transpose       :music-ops)
  (add-node-class 'flip            :music-ops)
  (add-node-class 'reverse         :music-ops)
  (add-node-class 'velocity        :music-ops)
  (add-node-class 'creschendo      :music-ops)
  (add-node-class 'iconic-sound    :notes)
  (labels ((add-note (name pitch duration)
             (add-node-copy-class
              name
              'iconic-sound
              `(:arguments ,(make-fmp (make-sound pitch 1.0 duration)))
              :notes)))
    (add-note "do"  #_do  1/4)
    (add-note "re"  #_re  1/4)
    (add-note "mi"  #_mi  1/4)
    (add-note "fa"  #_fa  1/4)
    (add-note "sol" #_so 1/4)
    (add-note "la"  #_la  1/4)
    (add-note "si"  #_si  1/4))
  (add-node-class 'iconic-interval :misc)
  (add-node-class 'iconic-rest     :misc)
  (add-node-class 'fmp-duration    :misc)
  (add-node-class 'iconic-number   :misc)
  (add-node-class 'iconic-add      :misc)
  (add-node-class 'iconic-multiply :misc)
  (add-node-class 'iconic-factor   :misc))

(defmethod FRAME-TYPE ((type (eql :music)))
  'music-frame)

(defmethod PANE-TYPE ((type (eql :music)))
  'music-doc)


;;;-------------------------------------------------------------------------;;;
;;;  D O N E !                                                              ;;;
;;;-------------------------------------------------------------------------;;;

(provide :MUSIC)

